/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "pan.h"

//=========================================================================

typedef struct
{
	pan_db db;
	gchar* filename;
	const server_data* server;
	const group_data* gdata;
}
AHDB;

//=========================================================================

static gchar*
get_filename (
	const server_data* s,
	const group_data* g )
{
	return g_strdup_printf ( "/%s/%s/%s.db", data_dir, s->name, g->name );
}

article_db
ahdb_ref (
	const server_data* s,
	const group_data* g )
{
	AHDB* ahdb = g_malloc0 ( sizeof(AHDB) );
	ahdb->filename = get_filename ( s, g );
	ahdb->db = pan_db_ref ( ahdb->filename );
	ahdb->server = s;
	ahdb->gdata = g;
	return (article_db)ahdb;
}

gboolean
ahdb_exists (
	const server_data* s,
	const group_data* g )
{
	gchar* filename = get_filename ( s, g );
	gboolean exists = g_file_exists ( filename );
	g_free ( filename );
	return exists;
}

void
ahdb_delete_db_file (
	const server_data* s,
	const group_data* g )
{
	gchar* filename = get_filename ( s, g );
	remove ( filename );
	g_free ( filename );
}

void ahdb_unref (
	article_db db )
{
	AHDB* ahdb = (AHDB*)db;
	g_free ( ahdb->filename );
	ahdb->filename = 0;
	pan_db_unref ( ahdb->db );
	ahdb->db = 0;
	g_free ( ahdb );
}

//=========================================================================

void ahdb_save (
	article_db db,
	const article_data* adata,
	gboolean sync )
{
	AHDB* ahdb = (AHDB*)db;

	gchar* pch = g_strdup_printf ( "%d\n%d\n%d\n%d\n%d\n%s\n%s\n%s\n%s",
		(int) adata->date,
		adata->state & ~STATE_NEW,
		adata->linecount,
		adata->parts,
		adata->part,
		adata->number,
		adata->subject,
		adata->author,
		(adata->references ? adata->references : "") );

	pan_db_put_value_str ( ahdb->db, adata->message_id, pch );

	g_free ( pch );

	if ( sync )
		ahdb_sync ( db );
}

void
ahdb_save_all (
	article_db db,
	GSList* list,
	GFunc gfunc,
	gpointer user_data )
{
	while ( list != NULL )
	{
		article_data* adata = (article_data*) list->data;
		if ( gfunc )
			(*gfunc)(adata,user_data);
		ahdb_save ( db, adata, FALSE );
		list = list->next;
	}

	ahdb_sync ( db );
}

void
ahdb_sync (
	article_db db )
{
	AHDB* ahdb = (AHDB*)db;
	pan_db_sync ( ahdb->db );
}

//=========================================================================

static article_data*
article_deserialize (
	const group_data* gdata,
	const char* message_id,
	const char* db_text )
{
	gchar** sd;
	article_data* adata;

	if ( !gdata )
		return NULL;

	/* break up the db_text */
	g_assert ( message_id!=NULL );
	g_return_val_if_fail ( db_text!=NULL, NULL );
	sd = g_strsplit ( db_text, "\n", 9 );
	g_return_val_if_fail ( sd!=NULL, NULL );

	/* build the article data */
	adata = g_new0 ( article_data, 1 );
	adata->message_id = g_strdup ( message_id );
	g_assert ( sd[0] != NULL );
	adata->date = strtol ( sd[0], NULL, 10 );
	adata->state = strtol ( sd[1], NULL, 10 );
	adata->linecount = strtol ( sd[2], NULL, 10 );
	adata->parts = strtol ( sd[3], NULL, 10 );
	adata->part = strtol ( sd[4], NULL, 10 );
	adata->number = sd[5];
	adata->subject = sd[6];
	adata->author = sd[7];
	adata->references = sd[8];
	//adata->group = (group_data*)gdata;

	/* cleanup */
	g_free ( sd[0] );
	g_free ( sd[1] );
	g_free ( sd[2] );
	g_free ( sd[3] );
	g_free ( sd[4] );
	g_free ( sd );

	return adata;
}

article_data*
ahdb_get (
	article_db db,
	const char* message_id )
{
	AHDB* ahdb = (AHDB*)db;
	gchar* s = pan_db_get_value_str ( ahdb->db, message_id );
	article_data* a = article_deserialize ( ahdb->gdata, message_id, s );
	g_free ( s );
	return a;
}

typedef struct
{
	AHDB* ahdb;
	GSList* l;
	GFunc gfunc;
	gpointer user_data;
}
GetArticlesStruct;

static void
ahdb_get_all_dbfunc (
	const char* key,
	const char* val,
	gpointer user_data )
{
	GetArticlesStruct* gas = (GetArticlesStruct*) user_data;
	article_data* a = article_deserialize ( gas->ahdb->gdata, key, val );
	if ( a != NULL )
		gas->l = g_slist_prepend ( gas->l, a );
	if ( gas->gfunc )
		(*gas->gfunc)(a, gas->user_data);
}

GSList*
ahdb_get_all (
	article_db db,
	GFunc gfunc,
	gpointer user_data )
{
	AHDB* ahdb = (AHDB*)db;
	GetArticlesStruct gas;
	gas.l = NULL;
	gas.ahdb = ahdb;
	gas.gfunc = gfunc;
	gas.user_data = user_data;
	pan_db_foreach ( ahdb->db, ahdb_get_all_dbfunc, (gpointer)&gas );
	return gas.l;
}

//=========================================================================

void
ahdb_erase (
	article_db db,
	const article_data* adata )
{
	AHDB* ahdb = (AHDB*)db;
	pan_db_erase ( ahdb->db, adata->message_id );
}

void
ahdb_erase_all (
	article_db db )
{
	AHDB* ahdb = (AHDB*)db;
	pan_db_erase_all ( ahdb->db );
}

