/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*********************
**********************  Includes
*********************/

#include "pan.h"

#include "xpm/delete.xpm"
#include "xpm/arrow_left.xpm"
#include "xpm/arrow_right.xpm"
#include "xpm/arrow_right_create.xpm"

/*********************
**********************  Defines / Enumerated types
*********************/

/*********************
**********************  Macros
*********************/

/*********************
**********************  Structures / Typedefs
*********************/

typedef struct
{
	int unique_id;
	MessageFilterType type;
	gchar* key;
	time_t last_match;
	guint16 onmatch_flags;
	int lifespan_days;
	gchar* group_regexp;
}
rule_t;

typedef struct
{
	GnomeDialog *dialog;

	GtkWidget* track_thread_rb;
	GtkWidget* track_thread_entry;
	GtkWidget* track_subject_rb;
	GtkWidget* track_subject_entry;
	GtkWidget* track_author_rb;
	GtkWidget* track_author_entry;

	GtkWidget* group_group_rb;
	GtkWidget* group_group_entry;
	GtkWidget* group_all_rb;

	GtkWidget* onmatch_expand_check;
	GtkWidget* onmatch_kill_check;

	GtkAdjustment* lifespan_days_data;
	GtkWidget* lifespan_days_rb;
	GtkWidget* lifespan_days_sb;
	GtkWidget* lifespan_immortal_rb;

	GtkWidget* rules_list;

	GnomePixmap* edit_pixmap;
	GtkWidget* edit_button;
	GnomePixmap* save_change_pixmap;
	GtkWidget* save_change_button;
	GnomePixmap* save_new_pixmap;
	GtkWidget* save_new_button;
	GnomePixmap* delete_pixmap;
	GtkWidget* delete_button;

	GtkTooltips* tips;
}
MessageFilterUI;

/*********************
**********************  Private Function Prototypes
*********************/

/* ui */
static GtkWidget* create_filter_page ( MessageFilterUI* );
static GtkWidget* create_track_page ( MessageFilterUI* ); 
static GtkWidget* create_group_page ( MessageFilterUI* ); 
static GtkWidget* create_onmatch_page ( MessageFilterUI* ); 
static GtkWidget* create_until_page ( MessageFilterUI* );

static void rule_ui_list_row_add ( MessageFilterUI*, const rule_t* );
static void rule_ui_list_row_refresh ( MessageFilterUI*, const rule_t* );
static void rule_ui_list_build ( MessageFilterUI* );
static rule_t* rule_ui_list_get_selected_rule ( MessageFilterUI* );
static gchar** rule_ui_list_get_text ( const rule_t* );

static void rule_to_ui ( const rule_t*, MessageFilterUI* );
static void ui_to_rule ( const MessageFilterUI*, rule_t* );

static void delete_button_clicked ( GtkButton*, gpointer );
static void save_change_button_clicked ( GtkButton*, gpointer );
static void edit_button_clicked ( GtkButton*, gpointer );
static void dialog_clicked_cb ( GnomeDialog*, gint, gpointer );

/* database */
static pan_db rule_db_open ( );
static void rule_db_save ( const rule_t* );
static void rule_db_save_new ( rule_t* );
static void rule_db_delete ( rule_t* );

/* utility */
static void rule_deserialize ( rule_t*, const char* key, const char* val );
static void rule_free ( rule_t* );

/* init */
static void message_filter_init ( void );
static void message_filter_init_foreach ( const char* key, const char* val, gpointer );

/*********************
**********************  Constants
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

static const int cols = 5;
static const char* SEQUENCE_PROPERTY = "unique_id_sequence";

/*********************
**********************  Variables 
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

static MessageFilterUI* _ui = NULL;

static gboolean _inited = FALSE;
static GSList* _rules = NULL;

/*********************
**********************  BEGINNING OF SOURCE
*********************/

/************
*************  PUBLIC ROUTINES
************/

MessageFilterAction
message_filter_get_action (
	const article_data* adata )
{
	printf ( "FIXME: message_filter_get_action\n" );
//        MESSAGE_FILTER_ACTION_NONE      = 0,
 //       MESSAGE_FILTER_ACTION_DELETE    = (1<<0),
  //      MESSAGE_FILTER_ACTION_EXPAND    = (1<<1)
	return MESSAGE_FILTER_ACTION_NONE;
}

void
message_filter_killfile_add (
	MessageFilterType type,
	const article_data* adata )
{
	g_return_if_fail ( adata!=NULL );
}

void
message_filter_tracker_add (
	MessageFilterType type,
	const article_data* adata )
{
	g_return_if_fail ( adata!=NULL );
}

static void
dialog_clicked_cb (
	GnomeDialog* dialog,
	gint button_index,
	gpointer user_data )
{
	MessageFilterUI* ui = (MessageFilterUI*) user_data;
	if ( button_index==0 )
	{
                gtk_widget_destroy ( GTK_WIDGET(ui->dialog) );
	}
}

static void
dialog_destroy_cb (
	GtkObject* object,
	gpointer user_data )
{
        gtk_widget_destroy ( GTK_WIDGET(_ui->dialog) );
	g_free ( _ui );
        _ui = NULL;
}


void
message_filter_ui ( void )
{
	message_filter_init ( );

	if ( _ui != NULL ) {
                if ( !GTK_WIDGET_MAPPED(GTK_WIDGET(_ui->dialog)) )
                        gtk_widget_map(GTK_WIDGET(_ui->dialog));
		gdk_window_raise ( GTK_WIDGET(_ui->dialog)->window );
		return;
	}

	_ui = (MessageFilterUI*) g_malloc0 ( sizeof(MessageFilterUI) );
	_ui->tips = gtk_tooltips_new ( );
	_ui->dialog = GNOME_DIALOG( gnome_dialog_new ( _("Message Filter"), GNOME_STOCK_BUTTON_OK, NULL ) );
        gtk_box_pack_start ( GTK_BOX(_ui->dialog->vbox), create_filter_page(_ui), TRUE, TRUE, 0 );
        gtk_signal_connect ( GTK_OBJECT(_ui->dialog), "clicked", dialog_clicked_cb, _ui );
        gtk_signal_connect ( GTK_OBJECT(_ui->dialog), "destroy", dialog_destroy_cb, _ui );

	rule_ui_list_build ( _ui );

	if ( Pan.active_group )
	{
		gtk_entry_set_text ( GTK_ENTRY(_ui->group_group_entry), Pan.active_group->name );
		if ( Pan.active_message_id )
		{
			gchar *author, *pch;

			article_db db = ahdb_ref ( Pan.active_server, Pan.active_group );
			article_data* adata = ahdb_get ( db, Pan.active_message_id );
			ahdb_unref ( db );

			author = adata->author;
			pch = strrchr ( author, '<' );
			if ( pch != NULL ) {
				g_free ( author );
				author = g_strdup ( pch+1 );
				pch = strrchr ( author, '>' );	
				if ( pch )
					*pch = '\0';
			}

			gtk_entry_set_text ( GTK_ENTRY(_ui->track_thread_entry), adata->message_id );
			gtk_entry_set_text ( GTK_ENTRY(_ui->track_subject_entry), adata->subject );
			gtk_entry_set_text ( GTK_ENTRY(_ui->track_author_entry), author );

			article_free ( adata );
			g_free ( author );
		}
	}

	gui_popup_draw ( GTK_WIDGET ( _ui->dialog ) );
}

/************
*************  PRIVATE ROUTINES
************/

/**
*** DATABASE
**/

static pan_db
rule_db_open ( void )
{
	gchar* filename = g_strdup_printf ( "/%s/killfile.db", data_dir );
	pan_db db = pan_db_open ( filename );	
	g_free ( filename );
	return db;
}

static void
rule_db_save (
	const rule_t* rule )
{
	/* setup */
	pan_db db = rule_db_open ( );
	char key[16];
	gchar* val;

	/* store the rule */
	sprintf ( key, "%d", rule->unique_id );
	val = g_strdup_printf ( "%d\n%d\n%s\n%lu\n%u\n%d\n%s",
		rule->unique_id,
		rule->type,
		rule->key,
		rule->last_match,
		rule->onmatch_flags,
		rule->lifespan_days,
		rule->group_regexp );
	pan_db_put_value_str ( db, key, val );

	/* cleanup */
	g_free ( val );
	pan_db_close ( db );
}

static void
rule_db_save_new (
	rule_t* rule )
{
	/* update sequence number */
	pan_db db = rule_db_open ( );
	rule->unique_id = pan_db_get_value_i ( db, SEQUENCE_PROPERTY );
	pan_db_add_value_i ( db, SEQUENCE_PROPERTY, 1 );
	pan_db_close ( db );

	/* save  */
	rule_db_save ( rule );
}

static void
rule_db_delete (
	rule_t* rule )
{
	pan_db db = rule_db_open ( );
	char key[16];
	sprintf ( key, "%d", rule->unique_id );
	pan_db_erase ( db, key );
	pan_db_close ( db );
}

/**
*** UTIL
**/

static void
rule_free (
	rule_t* rule )
{
	g_free ( rule->key );
	g_free ( rule->group_regexp );
	g_free ( rule );
}

static void
rule_deserialize (
	rule_t* rule,
	const gchar* key,
	const gchar* val )
{
        char** sd = g_strsplit ( val, "\n", 7 );
	rule->unique_id = (int) strtol ( sd[0], NULL, 10 );
	rule->type = (MessageFilterType) strtol ( sd[1], NULL, 10 );
	rule->key = sd[2];
	rule->last_match = (time_t) strtol ( sd[3], NULL, 10 );
	rule->onmatch_flags = (guint16) strtol ( sd[4], NULL, 10 );
	rule->lifespan_days = (int) strtol ( sd[5], NULL, 10 );
	rule->group_regexp = sd[6];

	g_free ( sd[0] );
	g_free ( sd[1] );
	g_free ( sd[3] );
	g_free ( sd[4] );
	g_free ( sd[5] );
	g_free ( sd );
}

/**
*** INIT
**/

static void
message_filter_init_foreach (
	const char* key,
	const char* val,
	gpointer user_data )
{
	if ( strcmp ( SEQUENCE_PROPERTY, key ) ) 
	{
		rule_t* rule = g_new0 ( rule_t, 1 );
		rule_deserialize ( rule, key, val );
		_rules = g_slist_prepend ( _rules, rule );
	}
}

static void
message_filter_init ( void )
{
	if ( !_inited )
	{
		pan_db db = rule_db_open ( );
		pan_db_foreach ( db, message_filter_init_foreach, NULL );
		pan_db_close ( db );

		_inited = TRUE;
	}
}

/**
*** UI
**/

static rule_t*
rule_ui_list_get_selected_rule (
	MessageFilterUI* ui )
{
	int row_index;
	GList* list = GTK_CLIST(ui->rules_list)->selection;
	if ( !list )
		return NULL;
	g_return_val_if_fail ( (int)list->data>=0, NULL ); /* no data */
	row_index = GPOINTER_TO_INT ( GTK_CLIST(ui->rules_list)->selection->data );
	return (rule_t*) gtk_clist_get_row_data ( GTK_CLIST(ui->rules_list), row_index );
}

static void
edit_button_clicked (
	GtkButton* button,
	gpointer user_data )
{
	MessageFilterUI* ui = (MessageFilterUI*) user_data;
	rule_t* rule = rule_ui_list_get_selected_rule ( ui );
	if ( rule != NULL )
	{
		rule_to_ui ( rule, ui );
	}
}

static gchar**
rule_ui_list_get_text (
	const rule_t* rule )
{
	gchar** ppch = g_new0 ( gchar*, 6 );
	const gchar* pch = NULL;
	char day_str[16];

	/* type */
	if ( rule->type == MESSAGE_FILTER_TYPE_SUBJECT )
		pch = _("Subject");
	else if ( rule->type == MESSAGE_FILTER_TYPE_THREAD )
		pch = _("Thread");
	else // author
		pch = _("Author");
	ppch[0] = g_strdup ( pch );

	/* key */
	ppch[1] = g_strdup ( rule->key );

	/* group regexp */
	if ( !strcmp ( rule->group_regexp, "*" ) )
		pch = _("All Groups");
	else
		pch = rule->group_regexp;
	ppch[2] = g_strdup ( pch );

	/* action */
	if ( rule->onmatch_flags & MESSAGE_FILTER_ACTION_DELETE )
		pch = _( "Delete" );
	else
		pch = _( "Expand" );
	ppch[3] = g_strdup ( pch );

	/* lifespan */
	if ( rule->lifespan_days < 0 )
		pch = _("Never Expire");
	else {
		sprintf ( day_str, _("%d Days"), rule->lifespan_days );
		pch = day_str;
	}
	ppch[4] = g_strdup ( pch );

	/* end */
	ppch[5] = NULL;
	return ppch;
}

static void
rule_ui_list_build (
	MessageFilterUI* ui )
{
	GtkCList* list = GTK_CLIST ( ui->rules_list );
	GSList* l;

	gtk_clist_freeze ( list );
	gtk_clist_clear ( list );
	for ( l=_rules; l!=NULL; l=l->next )
		rule_ui_list_row_add ( ui, (const rule_t*)l->data );
	gtk_clist_thaw ( list );
}

static void
rule_ui_list_row_add (
	MessageFilterUI* ui,
	const rule_t* rule )
{
	GtkCList* list = GTK_CLIST ( ui->rules_list );

	/* add the row */
	gchar** ppch = rule_ui_list_get_text ( rule );
	int row = gtk_clist_append ( list, ppch );
	gtk_clist_set_row_data ( list, row, (gpointer)rule );

	/* cleanup */
	g_strfreev ( ppch );
}

static void
rule_ui_list_row_refresh (
	MessageFilterUI* ui,
	const rule_t* rule )
{
	GtkCList* list = GTK_CLIST ( ui->rules_list );
	gchar** ppch = rule_ui_list_get_text ( rule );
	gint row = gtk_clist_find_row_from_data ( list, (gpointer)rule );
	int i;

	gtk_clist_freeze ( list );
	for ( i=0; i!=cols; ++i )
		gtk_clist_set_text ( list, row, i, ppch[i] );
	gtk_clist_thaw ( list );

	g_strfreev ( ppch );
}

static void
save_new_button_clicked (
	GtkButton* button,
	gpointer user_data )
{
	MessageFilterUI* ui = (MessageFilterUI*) user_data;
	rule_t* rule = g_new0 ( rule_t, 1 );
	ui_to_rule ( ui, rule );

	rule_db_save_new ( rule );
	_rules = g_slist_prepend ( _rules, rule );
	rule_ui_list_row_add ( ui, rule );
}

static void
save_change_button_clicked (
	GtkButton* button,
	gpointer user_data )
{
	MessageFilterUI* ui = (MessageFilterUI*) user_data;
	rule_t* rule = rule_ui_list_get_selected_rule ( ui );
	if ( rule != NULL )
	{
		ui_to_rule ( ui, rule );
		rule_db_save ( rule );
		rule_ui_list_row_refresh ( ui, rule );
	}
}

static void
delete_button_clicked (
	GtkButton* button,
	gpointer user_data )
{
	MessageFilterUI* ui = (MessageFilterUI*) user_data;
	rule_t* rule = rule_ui_list_get_selected_rule ( ui );
	if ( rule != NULL )
	{
		int list_row;
		rule_db_delete ( rule );
		g_slist_remove ( _rules, rule );
		list_row = gtk_clist_find_row_from_data ( GTK_CLIST(ui->rules_list), (gpointer)rule );
		gtk_clist_remove ( GTK_CLIST(ui->rules_list), list_row );
	}
}


static GtkWidget*
create_filter_page (
	MessageFilterUI* ui )
{
	GtkWidget* hbox = gtk_hbox_new ( FALSE, 5 );
	GtkWidget* rule_box = gtk_vbox_new ( FALSE, 5 );
	GtkWidget* buttonbar = gtk_vbox_new ( TRUE, 5 );
	gchar* rule_list_titles [] = { "Type", "Key", "Groups", "Action", "Lifespan" };

        gtk_box_pack_start ( GTK_BOX(rule_box), create_track_page(ui), TRUE, TRUE, 0 );
        gtk_box_pack_start ( GTK_BOX(rule_box), create_group_page(ui), TRUE, TRUE, 0 );
        gtk_box_pack_start ( GTK_BOX(rule_box), create_onmatch_page(ui), TRUE, TRUE, 0 );
        gtk_box_pack_start ( GTK_BOX(rule_box), create_until_page(ui), TRUE, TRUE, 0 );
	gtk_box_pack_start ( GTK_BOX(hbox), rule_box, TRUE, TRUE, 0  );

	/* edit buttons */
        ui->edit_pixmap = (GnomePixmap*) gnome_pixmap_new_from_xpm_d ( arrow_left_xpm );
	ui->edit_button = gtk_button_new ( );
	gtk_tooltips_set_tip ( ui->tips, ui->edit_button, _("Edit Rule"), NULL );
	gtk_container_add ( GTK_CONTAINER(ui->edit_button), GTK_WIDGET(ui->edit_pixmap) );
        ui->save_change_pixmap = (GnomePixmap*) gnome_pixmap_new_from_xpm_d ( arrow_right_xpm );
	ui->save_change_button = gtk_button_new ( );
	gtk_tooltips_set_tip ( ui->tips, ui->save_change_button, _("Save Changed Rule"), NULL );
	gtk_container_add ( GTK_CONTAINER(ui->save_change_button), GTK_WIDGET(ui->save_change_pixmap) );
        ui->save_new_pixmap = (GnomePixmap*) gnome_pixmap_new_from_xpm_d ( arrow_right_create_xpm );
	ui->save_new_button = gtk_button_new ( );
	gtk_tooltips_set_tip ( ui->tips, ui->save_new_button, _("Save New Rule"), NULL );
	gtk_container_add ( GTK_CONTAINER(ui->save_new_button), GTK_WIDGET(ui->save_new_pixmap) );
	gtk_box_pack_start ( GTK_BOX(buttonbar), ui->edit_button, FALSE, FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX(buttonbar), ui->save_change_button, FALSE, FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX(buttonbar), ui->save_new_button, FALSE, FALSE, 5 );
	gtk_box_pack_start ( GTK_BOX(hbox), buttonbar, TRUE, TRUE, 0  );
        gtk_signal_connect (GTK_OBJECT (ui->edit_button), "clicked", GTK_SIGNAL_FUNC(edit_button_clicked), ui );
        gtk_signal_connect (GTK_OBJECT (ui->save_change_button), "clicked", GTK_SIGNAL_FUNC(save_change_button_clicked), ui );
        gtk_signal_connect (GTK_OBJECT (ui->save_new_button), "clicked", GTK_SIGNAL_FUNC(save_new_button_clicked), ui );
	/* FIXME: sensitivity */
	/* FIXME: disabled hint in button */

	/* rules list */
	ui->rules_list = gtk_clist_new_with_titles ( cols, rule_list_titles );
        gtk_clist_set_column_width ( GTK_CLIST(ui->rules_list), 0, 60 );
        gtk_clist_set_column_width ( GTK_CLIST(ui->rules_list), 1, 150 );
        gtk_clist_set_column_width ( GTK_CLIST(ui->rules_list), 2, 150 );
        gtk_clist_set_column_width ( GTK_CLIST(ui->rules_list), 3, 100 );
        gtk_clist_set_column_width ( GTK_CLIST(ui->rules_list), 4, 80 );
	gtk_box_pack_start ( GTK_BOX(hbox), ui->rules_list, TRUE, TRUE, 0  );
	
	/* FIXME: events */

	/* delete rule */
	ui->delete_pixmap = (GnomePixmap*) gnome_pixmap_new_from_xpm_d ( delete_xpm );
	ui->delete_button = gtk_button_new ( );
	gtk_tooltips_set_tip ( ui->tips, ui->delete_button, _("Delete Selected Rule"), NULL );
	gtk_container_add ( GTK_CONTAINER(ui->delete_button), GTK_WIDGET(ui->delete_pixmap) );
	buttonbar = gtk_vbox_new ( FALSE, 5 );
	gtk_box_pack_end ( GTK_BOX(buttonbar), ui->delete_button, FALSE, FALSE, 5 );
	gtk_box_pack_start ( GTK_BOX(hbox), buttonbar, TRUE, TRUE, 0 );
        gtk_signal_connect (GTK_OBJECT (ui->delete_button), "clicked", GTK_SIGNAL_FUNC(delete_button_clicked), ui );


	return hbox;
}

static void
track_changed_cb (
	GtkToggleButton* tb,
	gpointer user_data )
{
}

static GtkWidget*
create_track_page (
	MessageFilterUI* ui )
{
        GtkWidget* frame = gtk_frame_new (_("Apply rule to articles matching"));
        GtkWidget* table = gtk_table_new ( 3, 3, FALSE );

	/* create widgetry... */
	ui->track_thread_rb = gtk_radio_button_new_with_label ( NULL, N_("Thread") );
	gtk_tooltips_set_tip ( ui->tips, ui->track_thread_rb, _("Apply to this Message ID and its followups"), NULL );
	ui->track_thread_entry = gtk_entry_new ( );
	ui->track_subject_rb = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(ui->track_thread_rb), N_("Subject") );
	gtk_tooltips_set_tip ( ui->tips, ui->track_subject_rb, _("Apply to messages with this subject"), NULL );
	ui->track_subject_entry = gtk_entry_new ( );
	ui->track_author_rb = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(ui->track_thread_rb), N_("Author") );
	gtk_tooltips_set_tip ( ui->tips, ui->track_author_rb, _("Apply to by this author"), NULL );
	ui->track_author_entry = gtk_entry_new ( );

	/* layout */
	gtk_table_attach_defaults ( GTK_TABLE(table), ui->track_subject_rb,    0, 1, 0, 1 );
	gtk_table_attach_defaults ( GTK_TABLE(table), ui->track_subject_entry, 1, 2, 0, 1 );
	gtk_table_attach_defaults ( GTK_TABLE(table), ui->track_author_rb,     0, 1, 1, 2 );
	gtk_table_attach_defaults ( GTK_TABLE(table), ui->track_author_entry,  1, 2, 1, 2 );
	gtk_table_attach_defaults ( GTK_TABLE(table), ui->track_thread_rb,     0, 1, 2, 3 );
	gtk_table_attach_defaults ( GTK_TABLE(table), ui->track_thread_entry,  1, 2, 2, 3 );
	gtk_container_add ( GTK_CONTAINER(frame), table );

	/* events */
        gtk_signal_connect (GTK_OBJECT (ui->track_subject_rb), "toggled", GTK_SIGNAL_FUNC(track_changed_cb), ui );
        gtk_signal_connect (GTK_OBJECT (ui->track_author_rb), "toggled", GTK_SIGNAL_FUNC(track_changed_cb), ui );
        gtk_signal_connect (GTK_OBJECT (ui->track_thread_rb), "toggled", GTK_SIGNAL_FUNC(track_changed_cb), ui );

	return frame;
}

static GtkWidget*
create_group_page (
	MessageFilterUI* ui )
{
        GtkWidget* frame = gtk_frame_new (_("Apply Rule in Groups"));
        GtkWidget* table = gtk_table_new ( 2, 2, FALSE );

	/* create widgetry... */
	ui->group_group_rb = gtk_radio_button_new_with_label ( NULL, N_("Group") );
	ui->group_group_entry = gtk_entry_new ( );
	ui->group_all_rb = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(ui->group_group_rb), N_("All Groups") );

	/* layout */
	gtk_table_attach_defaults ( GTK_TABLE(table), ui->group_group_rb, 0, 1, 0, 1 );
	gtk_table_attach_defaults ( GTK_TABLE(table), ui->group_group_entry, 1, 2, 0, 1 );
	gtk_table_attach_defaults ( GTK_TABLE(table), ui->group_all_rb, 0, 2, 1, 2 );
	gtk_container_add ( GTK_CONTAINER(frame), table );

	/* events */
	/* FIXME */

	return frame;
}

static GtkWidget*
create_onmatch_page (
	MessageFilterUI* ui )
{
        GtkWidget* frame = gtk_frame_new (_("Action to Take"));
        GtkWidget* box = gtk_vbox_new ( TRUE, 5 );

	/* widgets */
	ui->onmatch_expand_check = gtk_check_button_new_with_label ( _("Expand thread in article tree to make message visible") );
	ui->onmatch_kill_check = gtk_check_button_new_with_label ( _("Delete message") );

	/* layout */
	gtk_box_pack_start ( GTK_BOX(box), ui->onmatch_expand_check, TRUE, TRUE, 0 );
	gtk_box_pack_start ( GTK_BOX(box), ui->onmatch_kill_check, TRUE, TRUE, 0 );
	gtk_container_add ( GTK_CONTAINER(frame), box );

	/* events */
	/* FIXME */

	return frame;
}

static GtkWidget*
create_until_page (
	MessageFilterUI* ui )
{
        GtkWidget* frame = gtk_frame_new (_("Lifespan"));
        GtkWidget* vbox = gtk_vbox_new ( FALSE, 2 );
        GtkWidget* hbox = gtk_hbox_new ( FALSE, 2 );
	GtkWidget* lifespan_days_lb;

	/* widgets */
	ui->lifespan_days_rb = gtk_radio_button_new_with_label ( NULL, _("Delete rule after") );
	ui->lifespan_days_data = GTK_ADJUSTMENT ( gtk_adjustment_new ( 7, 1, 90, 1, 7, 30 ) );
	ui->lifespan_days_sb = gtk_spin_button_new ( ui->lifespan_days_data, 1, 0 );
	lifespan_days_lb = gtk_label_new ( _( "days since last match" ) );
	ui->lifespan_immortal_rb = gtk_radio_button_new_with_label_from_widget ( GTK_RADIO_BUTTON(ui->lifespan_days_rb), _("Keep this rule forever") );

	/* layout */
	gtk_box_pack_start ( GTK_BOX(hbox), ui->lifespan_days_rb, FALSE, FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX(hbox), ui->lifespan_days_sb, FALSE, FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX(hbox), lifespan_days_lb, FALSE, FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX(vbox), hbox, FALSE, FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX(vbox), ui->lifespan_immortal_rb, FALSE, FALSE, 0 );
	gtk_container_add ( GTK_CONTAINER(frame), vbox );

	return frame;
}

static void
rule_to_ui (
	const rule_t* rule,
	MessageFilterUI* ui )
{
	/* type */
	gtk_entry_set_text ( GTK_ENTRY(ui->track_subject_entry), "" );
	gtk_entry_set_text ( GTK_ENTRY(ui->track_thread_entry), "" );
	gtk_entry_set_text ( GTK_ENTRY(ui->track_author_entry), "" );
	switch ( rule->type )
	{
		case MESSAGE_FILTER_TYPE_SUBJECT:
			gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(ui->track_subject_rb), TRUE );
			gtk_entry_set_text ( GTK_ENTRY(ui->track_subject_entry), rule->key );
			break;
		case MESSAGE_FILTER_TYPE_THREAD:
			gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(ui->track_thread_rb), TRUE );
			gtk_entry_set_text ( GTK_ENTRY(ui->track_thread_entry), rule->key );
			break;
		case MESSAGE_FILTER_TYPE_AUTHOR:
			gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(ui->track_author_rb), TRUE );
			gtk_entry_set_text ( GTK_ENTRY(ui->track_author_entry), rule->key );
			break;
		default:
                        g_assert_not_reached ( );
	}

	/* scope */
	if ( strcmp ( rule->group_regexp, "*" ) ) {
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(ui->group_group_rb), TRUE );
		gtk_entry_set_text ( GTK_ENTRY(ui->group_group_entry), rule->group_regexp );
	} else {
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(ui->group_all_rb), TRUE );
		gtk_entry_set_text ( GTK_ENTRY(ui->group_group_entry), "" );
	}

	/* onmatch */
	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(ui->onmatch_expand_check), rule->onmatch_flags & MESSAGE_FILTER_ACTION_EXPAND );
	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(ui->onmatch_kill_check), rule->onmatch_flags & MESSAGE_FILTER_ACTION_DELETE );

	/* lifespan */
	if ( rule->lifespan_days>0 ) {
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(ui->lifespan_days_rb), TRUE );
		gtk_spin_button_set_value ( GTK_SPIN_BUTTON(ui->lifespan_days_sb), (gfloat)rule->lifespan_days );
	} else {
		gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(ui->lifespan_immortal_rb), TRUE );
	}
}

static void
ui_to_rule (
	const MessageFilterUI* ui,
	rule_t* rule )
{
	/* type, key */
	g_free ( rule->key );
	if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(ui->track_thread_rb) ) )
	{
		rule->type = MESSAGE_FILTER_TYPE_THREAD;
		rule->key = gtk_editable_get_chars ( GTK_EDITABLE(ui->track_thread_entry), 0, -1 );
	}
	else if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(ui->track_subject_rb) ) )
	{
		rule->type = MESSAGE_FILTER_TYPE_SUBJECT;
		rule->key = gtk_editable_get_chars ( GTK_EDITABLE(ui->track_subject_entry), 0, -1 );
	}
	else // author rb
	{
		rule->type = MESSAGE_FILTER_TYPE_AUTHOR;
		rule->key = gtk_editable_get_chars ( GTK_EDITABLE(ui->track_author_entry), 0, -1 );
	}

	/* flags */
	rule->onmatch_flags = 0;
	if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(ui->onmatch_expand_check) ) )
		rule->onmatch_flags |= MESSAGE_FILTER_ACTION_EXPAND;
	if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(ui->onmatch_kill_check) ) )
		rule->onmatch_flags |= MESSAGE_FILTER_ACTION_DELETE;

	/* lifespan days */
	if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(ui->lifespan_days_rb) ) )
		rule->lifespan_days = gtk_spin_button_get_value_as_int ( GTK_SPIN_BUTTON(ui->lifespan_days_sb) );
	else // never expire
		rule->lifespan_days = -1;

	/* group */
	g_free ( rule->group_regexp );
	if ( gtk_toggle_button_get_active ( GTK_TOGGLE_BUTTON(ui->group_group_rb) ) )
		rule->group_regexp = gtk_editable_get_chars ( GTK_EDITABLE(ui->group_group_entry), 0, -1 );
	else
		rule->group_regexp = g_strdup ( "*" );
}
