/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <errno.h>
#include <ctype.h>

#include "pan.h"

int must_exit = FALSE;
int stop_all = FALSE;
int online = FALSE;

int online_prompt = 1;

pthread_mutex_t online_m = PTHREAD_MUTEX_INITIALIZER;
pthread_mutex_t offline_m = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t online_c = PTHREAD_COND_INITIALIZER;
pthread_cond_t offline_c = PTHREAD_COND_INITIALIZER;


char *
nntp_connect (void)
{
	char *buffer;

	if (Pan.active_server == NULL)
		return (_("There is no active server.  Please configure one."));
	if (tcp_open() < 0)
		return (_("tcp_open failed."));
	if ((buffer = get_server()) == NULL)
		return (_("Error reading from socket."));

	/* Now we attempt authentication.  This is not implemented 
	 * exactly as "Common NNTP Extensions" specifies, being that
	 * we didn't wait to be asked for it, but hopefully it 
	 * will work. */

	if (Pan.active_server->need_auth) {
		if (put_server_va ("AUTHINFO user %s\r\n", Pan.active_server->username) == -1)
			return (_("Error writing to socket."));
		if ((buffer = get_server()) == NULL)
			return (_("Error reading from socket."));
		if (atoi(buffer) == 381) {
			/* 381 Means: More authentication information required */
			if (put_server_va ("AUTHINFO pass %s\r\n", Pan.active_server->password) == -1)
				return (_("Error writing to socket."));
			if ((buffer = get_server()) == NULL)
				return (_("Error reading from socket."));
			if (atoi(buffer) != 281) {
				/* 281 Means: Authentication accepted */
				log_add_va (_("Authentication for server '%s' unsuccessful."), buffer, Pan.active_server->name);
				return (buffer);
			}
		}
		/* Else authentication not required or not understood */
	}
	if (put_server ("MODE READER\r\n") == -1)
		return (_("Error writing to socket."));
	if ((buffer = get_server()) == NULL)
		return (_("Error reading from socket."));
	
	if (atoi (buffer) == 200)
		Pan.active_server->posting = 1;
	else if (atoi (buffer) == 201)
		Pan.active_server->posting = 0;
	else 
		return (buffer);

	return (NULL);
}


/*---[ connect_dialog_clicked_cb ]------------------------------------
 * "connect?" dialog buttons were pressed, if they pressed OK, then
 * wake up the online thread, then destroy the dialog.
 *--------------------------------------------------------------------*/
void
connect_dialog_clicked_cb (GnomeDialog *dialog, int button)
{
	if (button == 0) {
		pthread_mutex_lock (&online_m);
  		pthread_cond_signal (&online_c);
		pthread_mutex_unlock (&online_m);
	}
	gtk_widget_destroy (GTK_WIDGET(dialog));
}


void
nntp_want_connect (void)
{
	GtkWidget *connect_dialog, *label;

	if (online == FALSE) {
		if (online_prompt) { /* FIXME: If prompt for connects */
			pan_lock ();
			connect_dialog = gnome_dialog_new (_("Go online?"), GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
			label = gtk_label_new (_("A connection to the server is needed.\nShall we? "));
			gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(connect_dialog)->vbox), label);
			gtk_signal_connect (GTK_OBJECT(connect_dialog), "clicked",
					    GTK_SIGNAL_FUNC(connect_dialog_clicked_cb), NULL);
			//gtk_window_set_modal (GTK_WINDOW(connect_dialog), TRUE);
			gui_popup_draw (connect_dialog);
			pan_unlock ();
		}
		else {
			pthread_mutex_lock (&online_m);
  			pthread_cond_signal (&online_c);
			pthread_mutex_unlock (&online_m);
		}
	}
}


void
nntp_want_disconnect (void)
{
	if (online == TRUE) {
		pthread_mutex_lock (&offline_m);
  		pthread_cond_signal (&offline_c);
		pthread_mutex_unlock (&offline_m);
	}
}

static
void
nntp_connection_manager (void)
{
	char *p;

	while (1) {
		/* Wait for a nntp_want_connect request */
		if (online == FALSE) {
			pthread_mutex_lock (&online_m);
			pthread_cond_wait (&online_c, &online_m);
			gui_set_status_va (_("Connecting to server %s"), Pan.active_server->name);
			if ((p = nntp_connect())) {
				gui_clean();
				pan_error_dialog (_("Could not connect to server:\n%s"), p);
				pthread_mutex_unlock (&online_m);
				continue;
			}
			online = TRUE;
			gui_set_status_default (_("Pan - Online"));
			gui_clean ();
			log_add (_("Pan - Online"));
			pthread_mutex_unlock (&online_m);
		}
		
		/* Wait for a nntp_want_disconnect request */
		if (online == TRUE) {
			pthread_mutex_lock (&offline_m);
			pthread_cond_wait (&offline_c, &offline_m);
			put_server ("QUIT\r\n");
			tcp_close ();
			must_exit = FALSE;
			if (stop_all) {
				queue_purge ();
				stop_all = FALSE;
			}	
			online = FALSE;
			gui_set_status_default (_("Pan - Offline"));
			gui_clean ();
			log_add (_("Pan - Offline"));
			pthread_mutex_unlock (&offline_m);
		}
	}
}


/* nntp_init(): This function is called at startup */
void
nntp_init (void)
{
	pthread_create (&Pan.conmgr_t, NULL, (void *) &nntp_connection_manager, NULL);
	pthread_detach (Pan.conmgr_t);
}


/**************************************************
* go on/off line code
***************************************************/
/*
void
connect_dialog_clicked_cb (GnomeDialog *dialog, int button)
{
	if (button == 0) {
		pthread_mutex_lock(&online_m);
		pthread_cond_broadcast(&online_c);
		pthread_mutex_unlock(&online_m);
	}
	gtk_widget_destroy (GTK_WIDGET(dialog));
}


void
go_online (void)
{
	GtkWidget *connect_dialog, *label;

	if (online == FALSE) {
		if (1) {	
			pan_lock ();
			connect_dialog = gnome_dialog_new (_("Go online?"),
							   GNOME_STOCK_BUTTON_YES,
							   GNOME_STOCK_BUTTON_NO, NULL);
			abel = gtk_label_new (_("A connection to the server is needed.\nShall we? "));
			gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(connect_dialog)->vbox), label);
	   		gtk_signal_connect (GTK_OBJECT(connect_dialog), "clicked",
					    GTK_SIGNAL_FUNC(connect_dialog_clicked_cb), NULL);
			gtk_window_set_modal (GTK_WINDOW(connect_dialog), TRUE);
			gui_popup_draw (connect_dialog);
			pan_unlock ();
		}
		else {
			if (nntp_connect())
				must_exit = TRUE;
		}
	}
	else {
		return;
	}
}


void
nntp_want_disconnect (void)
{
	if (online == TRUE) {
		put_server ("QUIT\r\n");
		tcp_close ();
		must_exit = FALSE;
		gui_set_status_default (_("Pan - Offline"));
		log_add (_("Pan - Offline"));
		gui_clean ();
		if (stop_all) {
			queue_purge ();
			stop_all = FALSE;
		}
		//pthread_mutex_lock(&online_m);
		online = FALSE;
		//pthread_mutex_unlock(&online_m);
	}
}
*/


int
is_online (void)
{
	return online;
}


void
network_error (const char *format, ...)
{
	va_list args;
	char *p = NULL;

	g_return_if_fail (format != NULL);

	va_start (args, format);
	p = g_strdup_vprintf (format, args);
	va_end (args);

	pan_lock ();
	gnome_error_dialog (p);
	pan_unlock ();
	g_free (p);
	nntp_want_disconnect ();
}


int
nntp_set_group (group_data *data)
{
	char *buffer;

	g_assert (data);

	if (put_server_va("GROUP %s\r\n", data->name) == -1) {
		return 0;
	}

	if ((buffer = get_server()) == NULL) {
		return 0;
	}

	if (atoi(buffer) != 211)
		return 1;
	else
		return 0;
}



/*---[ nntp_post ]----------------------------------------------------
 * post an article to the news server
 * @msg: ???
 * return value:
 *--------------------------------------------------------------------*/
int
nntp_post (char *msg)
{
	char *buffer;

	gui_set_status (_("Posting message..."));
	if (put_server ("POST\r\n") == -1) {
		network_error (_("Error writing to socket during post."));
		return -1;
	}
	if ((buffer = get_server()) == NULL) {
		network_error (_("Error reading from socket during post."));
		return -1;
	}
	if (atoi(buffer) == 340) {
		if (put_server_va ("%s\r\n.\r\n", msg) == -1) {
			network_error (_("Error writing to socket during post."));
			return -1;
		}
		if ((buffer = get_server()) == NULL) {
			network_error (_("Error reading from socket during post."));
			return -1;
		}
		if (atoi(buffer) == 240) {
			return 0;
		}
	}
	gui_clean ();
	pan_error_dialog (_("Error posting message.  Server said:\n'%s'"), buffer);
	return -1;
}


/***************************************************************
 * grouplist_download 
 ****************************************************************/
GSList *
nntp_grouplist_download (void)
{
	char *buffer = NULL;
	group_data *gdata = NULL;
	int count = 0;
	GSList *glist = NULL;

	gui_set_status (_("Retrieved 0 groups."));

	if (put_server("LIST active\r\n") == -1) {
		network_error (_("Error writing to socket."));
		return NULL;
	}
	if ( (buffer = get_server()) == NULL) {
		network_error (_("Error reading from socket."));
		return NULL;
	}
	if (atoi(buffer) != 215) {
		network_error (_("Group list could not be retrieved\nServer said: %s"), buffer);
		return NULL;
	} else {
		if ((buffer = get_server()) == NULL) {
			network_error (_("Error reading from socket."));
			return NULL;
		}
		gui_set_progress_activity (1);
		while (strncmp(buffer, ".\r\n", 3)) {
			if (must_exit) {
				grouplist_free (glist);
				nntp_want_disconnect();
				return NULL;
			}
			gdata = (group_data *) g_malloc0 (sizeof(group_data));
			gdata->name = g_strdup (strtok(buffer, " "));
			glist = g_slist_prepend (glist, gdata);
			++count;
			if ((count % 43) == 0) {
				gui_set_progress_activity ((gfloat)count );
				gui_set_status_va (_("Retrieved %d groups."), count);
			}
			if ((buffer = get_server()) == NULL) {
				network_error (_("Error reading from socket."));
				grouplist_free (glist);
				return NULL;
			}
		}
	}
//	gui_set_info_va (_("Retrieved %d groups."), count);
	log_add_va (_("Retrieved %d groups"), count);

	gui_set_status_va (_("Sorting %d groups."), count);
	glist = g_slist_sort ( glist, (GCompareFunc)compare_group_data );
	return glist;
}





/*---[ nntp_articlelist_download ]------------------------------------
 *
 *--------------------------------------------------------------------*/
GSList *
nntp_articlelist_download (group_data *gdata, int flags, int *count)
{
	char *buffer = NULL;
	article_data *adata = NULL;
	gint total, total_new;
	int steps = 0;
	int first = 0;
	int last = 0;
	int p_last = 0;
	int part = 0;
	int parts = 0;
	int i;
	time_t current_time, date;
	char **sd;
	char *s;
	GSList *alist = NULL;
	int expiration = gnome_config_get_int_with_default ("/Pan/Articles/Expiration=30", NULL);


        if (put_server_va("GROUP %s\r\n", gdata->name) == -1) {
                network_error(_("Error writing to socket."));
                return NULL;
        }
        if ((buffer = get_server()) == NULL) {
                network_error(_("Error reading from socket."));
                return NULL;
        }

        if (strtol(buffer, NULL, 10) != 211) {
                network_error(_("Couldn't select group '%s'"), gdata->name);
                return NULL;
        }

	sscanf (buffer, "%*d %d %d %d", &total, &first, &last);

	p_last = group_get_attrib_i (gdata, "Last");

	total_new = last - p_last;

	if (total == '0') {
//		gui_set_info (_("No articles"));
		log_add_va (_("No articles found for group '%s'"), gdata->name);
		return NULL;
	}
	if ((flags & QUEUE_ARTICLELIST_DOWNLOAD_NEW) && p_last) {
		if (p_last == last) {
//			gui_set_info (_("No new articles"));
			log_add_va (_("No new articles for group %s"), gdata->name);
			return NULL;
		}
		gui_set_status (_("Retrieving new article headers..."));
		if (put_server_va("XOVER %d-%d\r\n", p_last, last) == -1) {
			network_error (_("Error writing to socket."));
			return NULL;
		}
	}
	else {
		if ((total > 150) && ((flags & 0x003F) == QUEUE_ARTICLELIST_DOWNLOAD_SAMPLE)) {
			gui_set_status (_("Retrieving sample of article headers"));
			if (put_server_va ("XOVER %d-%d\r\n", first, first + 150) == -1) {
				network_error (_("Error writing to socket."));
				return NULL;
			}
		}
		else {
			gui_set_status (_("Retrieving all article headers..."));
			if (put_server_va ("XOVER %d-%d\r\n", first, last) == -1) {
				network_error (_("Error writing to socket."));
				return NULL;
			}
		}
	}

	if ((buffer = get_server()) == NULL) {
		network_error (_("Error reading from socket."));
		return NULL;
	}

	if (strtol(buffer, (char **) NULL, 10) != 224) {
		network_error (_("Couldn't retrieve headers\nServer said: %s"), buffer);
		return NULL;
	}

	time (&current_time);
	if ((buffer = get_server()) == NULL) {
		network_error (_("Error reading from socket."));
		return NULL;
	}

	steps = ((flags&QUEUE_ARTICLELIST_DOWNLOAD_NEW) && (group_get_attrib_i(gdata,"Total")>0)) ? total_new : total;
	gui_progress_init_steps ( steps );

	while (strncmp(buffer, ".\r\n", 3))
	{
		if (must_exit)
		{
			g_slist_foreach ( alist, (GFunc)article_free, NULL );
			nntp_want_disconnect ( );
			return NULL;
		}
		sd = g_strsplit (buffer, "\t", 0);
		date = parse_date (sd[3]);
		if ((current_time - date) > (86400 * expiration)) {
			g_strfreev (sd);
			if ((buffer = get_server()) == NULL) {
				network_error (_("Error reading from socket."));
				articlelist_free (alist);
				g_strfreev (sd);
				return NULL;
			}
			continue;
		}
		adata = (article_data *) g_malloc0(sizeof(article_data));
		adata->number = g_strdup (sd[0]);
		adata->subject = g_strdup (sd[1]);
		adata->author = g_strdup (sd[2]);
		adata->date = date;
		adata->message_id = g_strdup (sd[4]);
		adata->references = g_strdup (sd[5]);
		adata->linecount = strtol(sd[7], NULL, 10);
		//adata->group = gdata;
		adata->state = STATE_NEW;

		g_strfreev (sd);

		/* Here we look for the (n/N) or [n/N] construct in subject lines,
		 * starting at the end of the string and working backwards */
		part = 0;
		parts = 0;
		s = adata->subject;
		for (i = strlen(s) - 1; i > 0; --i) {
			if ((s[i] == ')') || (s[i] == ']')) {
				for (--i; isdigit((int)s[i]); --i);
				if (s[i] == '/') {
					parts = atoi(s + i + 1);
					for (--i; isdigit((int)s[i]); --i);
					if ((s[i] == '(') || (s[i] == '[')) {
						part = atoi(s + i + 1);
						break;
					}
				}
			}
		}

		/* Verify Multipart info */
		if ((parts > 1) && (part <= parts)) {
			/* We found one! */
			adata->parts = parts;
			adata->part = part;
		}
		else {
			adata->parts = 0;
			adata->part = 0;
		}
		/* Add the article to the article list */
		alist = g_slist_prepend (alist, adata);

		++*count;
		gui_progress_next_step ( );
		if ( ! ( *count % 29 ) )
			gui_set_status_va (_("Retrieving new article headers... %d of %d"), *count, steps );

		if ((buffer = get_server()) == NULL) {
			network_error (_("Error reading from socket."));
			articlelist_free (alist);
			return NULL;
		}
	}
	
	group_set_attrib_i (gdata, "Last", last);

	return alist;
}


/***************************************************************
* article_download 
***************************************************************/
int
nntp_article_download (group_data *gdata, article_data *adata)
{
	char *buffer;
	FILE *f;
	
	gui_set_status_va(_("Downloading article '%s'"), adata->subject);

	if (nntp_set_group(gdata)) {
		network_error(_("Couldn't select group '%s'"), gdata->name);
		return -1;
	}

	if (put_server_va ("HEAD %s\r\n", adata->number) == -1) {
		network_error (_("Error writing to socket."));
		return -1;
	}
	if ((buffer = get_server()) == NULL) {
		network_error (_("Error reading from socket."));
		return -1;
	}
		
	if (strtol(buffer, (char **) NULL, 10) != 221) {
		return -1;
	}
	if ((buffer = get_server()) == NULL) {
		network_error (_("Error reading from socket."));
		return -1;
	}
	
	f = acache_open_header (adata, ACACHE_NONE);
	while (strncmp(buffer, ".\r\n", 3)) {
		if (must_exit) {
			acache_close_header (f);
			acache_delete (adata->message_id);
			nntp_want_disconnect ();
			return -1;
		}
		acache_putline_header (f, buffer);
		if ((buffer = get_server()) == NULL) {
			network_error (_("Error reading from socket."));
			acache_close_header (f);
			acache_delete (adata->message_id);
			return -1;
		}
	}
	acache_close_header (f);

	if (put_server_va ("BODY %s\r\n", adata->number) == -1) {
		network_error (_("Error writing to socket."));
		return -1;
	}
	if ((buffer = get_server()) == NULL) {
		network_error (_("Error reading from socket."));
		return -1;
	}
	if (strtol(buffer, (char **) NULL, 10) != 222) {
		return -1;
	}
	
	if ((buffer = get_server()) == NULL) {
		network_error (_("Error reading from socket."));
		return -1;
	}
	f = acache_open_body (adata, ACACHE_NONE);
	gui_progress_init_steps ( adata->linecount );
	while (strncmp(buffer, ".\r\n", 3)) {
		if (must_exit) {
			acache_close_body (f);
			acache_delete (adata->message_id);
			nntp_want_disconnect ();
			return -1;
		}
		acache_putline_body (f, buffer);
		gui_progress_next_step ( );
		if ((buffer = get_server()) == NULL) {
			network_error (_("Error reading from socket."));
			acache_close_body (f);
			acache_delete (adata->message_id);
			return -1;
		}
	}
	acache_close_body (f);
	gui_set_progress (1);

	return 0;
}


/***************************************************************
* article_decode 
***************************************************************/
int
nntp_article_decode (
	group_data* gdata,
	GSList* list_of_msg_id )
{
	char *buffer;
	float count = 0;
	GSList *l;
	FILE *f;
	article_data *adata0;

	/* Set the group */
	if ( nntp_set_group ( gdata ) )
	{
		network_error ( _("Couldn't select group '%s'"), gdata->name );
		return -1;
	} 

	/* Download bodies for all of the parts */
	for ( l=list_of_msg_id; l!=NULL; l=l->next)
	{
		/* get the header information */
		article_db db = ahdb_ref ( Pan.active_server, gdata );
		const gchar* message_id = (const gchar*) l->data;
		adata0 = ahdb_get ( db, message_id );
		ahdb_unref ( db );

		/* if we couldn't get it, fail. */
		if ( !adata0 ) {
			log_add_va (_("Couldn't find article header for message_id: %s"), l->data);
			//FIXME article_add_flag (adata, STATE_ERROR, TRUE);
			return -1;
		} 
	
		if (adata0->parts > 1)
			gui_set_status_va (_("Downloading part %d of %d: %s"), adata0->part, adata0->parts, adata0->subject);
		else
			gui_set_status_va (_("Downloading: %s"), adata0->subject);
		gui_set_progress (0);

		if (put_server_va("BODY %s\r\n", adata0->number) == -1) {
			network_error(_("Error writing to from socket."));
			article_free (adata0);
			return -1;
		}
		if ((buffer = get_server()) == NULL) {
			network_error(_("Error reading from socket."));
			article_free (adata0);
			return -1;
		}

		if (strtol(buffer, (char **) NULL, 10) != 222) {
			article_add_flag (adata0, STATE_ERROR, TRUE);
			article_free (adata0);
			return -1;
		}
		count = 1;
		if ((buffer = get_server()) == NULL) {
			network_error (_("Error reading from socket."));
			article_free (adata0);
			return -1;
		}
		
		f = acache_open_body (adata0, ACACHE_KEEP);
		while (strncmp(buffer, ".\r\n", 3)) {
			if (must_exit) {
				acache_close_body (f);
				//FIXME acache_delete (adata->message_id);
				article_free (adata0);
				return -1;
			}
			acache_putline_body (f, buffer);
			if ((buffer = get_server()) == NULL) {
				network_error (_("Error reading from socket."));
				acache_close_body (f);
				//FIXME acache_delete (adata->message_id);
				article_free (adata0);
				return -1;
			}
			gui_set_progress ((gfloat) count++ / (gfloat) adata0->linecount);
		}
		acache_close_body (f);
		article_free (adata0);
	}

	gui_set_progress (1);
	return 0;
}


void
nntp_send_noop (void)
{
	char *buffer;

	if (put_server("HELP\r\n") == -1) {
		network_error (_("Error writing to socket during HELP no-op."));
		return;
	}
	if ((buffer = get_server()) == NULL) {
		network_error (_("Error reading from socket during HELP no-op."));
		return;
	}
	if (atoi(buffer) == 100) {
 		while (strncmp(buffer, ".\r\n", 3)) {
			if ((buffer = get_server()) == NULL) {
				network_error (_("Error reading from socket during HELP no-op."));
				return;
			}
		}
	}
}


/*
 * Menu callback functions 
void
nntp_want_disconnect_mf (void)
{
	if (is_online()) {
		if (queue_active(-1) == 1)
			must_exit = TRUE;
		else
			nntp_want_disconnect();
	}
}
*/

void
stop_all_mf (void)
{
	if (is_online()) {
		if (queue_active(-1) == 1) {
			must_exit = TRUE;
			stop_all = TRUE;
	 	}
		else {
			/* FIXME: Purge queue */	
			nntp_want_disconnect();
		}
	}
}
