/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "pan.h"

PanPrefsWindow *prefs_window = NULL;
GSList *servers;

/* Items that we will keep in memory, since
 * they might be looked up frequently */
gchar *temp_dir = NULL;
gchar *data_dir = NULL;
gchar *download_dir = NULL;

gchar *grouplist_font = NULL;
gchar *articlelist_font = NULL;
gchar *message_body_font = NULL;

static GtkWidget* prefs_general_page ( void );

#define DISPLAY_PREFS_PREVIEW_FONT_SIZE 12

/*--------------------------------------------------------------------
 * builds the server GSList from the data in the config files
 *--------------------------------------------------------------------*/
void
load_servers (void)
{
	char **sd;
	char *p;
	int i;
	server_data *sdata;

	if (servers) {
		g_slist_foreach (servers, (GFunc) server_free, NULL);
		g_slist_free (servers);
		servers = NULL;
	}
	if ((p = gnome_config_get_string ("/Pan/Servers/Names")) == NULL)
		return;

	sd = g_strsplit (p, ",", 0);
	g_free (p);
	if (sd == NULL) {
		g_error ("Error in config file.");
		return;
	}
	for (i = 0; sd[i]; i++) {
		gchar* buf = g_strdup_printf ( "/Pan/%s/", sd[i] );
		if (gnome_config_has_section (buf)) {
			gnome_config_push_prefix (buf);
			sdata = (server_data *) g_malloc0(sizeof(server_data));
			if ((sdata->address = gnome_config_get_string ("Address")) == NULL) {
				g_free (sdata);
				g_free (buf);
				break;
			}
			sdata->name = g_strdup (sd[i]);
			sdata->port = gnome_config_get_string ("Port=119");
			sdata->need_auth = gnome_config_get_int ("Auth_Required=0");
			sdata->gen_msgid = gnome_config_get_int ("Generate_Msgid=0");
			sdata->username = gnome_config_get_string ("Username");
			sdata->password = gnome_config_get_string ("Password");
			servers = g_slist_append (servers, sdata);
			gnome_config_pop_prefix ();
		}
		g_free(buf);
	}

	g_strfreev (sd);
}


void 
save_servers (void)
{
	server_data *sdata = NULL;
	GSList *l = NULL;
	int i = 0;
	char *p = NULL;
	char **sd = NULL;

	/* Out with the old */
	if ((p = gnome_config_get_string ("/Pan/Servers/Names"))) {
		sd = g_strsplit (p, ",", -1);
		for (i = 0; sd[i]; i++) {
			gchar* buf = g_strdup_printf ( "/Pan/%s", sd[i] );
			gnome_config_clean_section (buf);
			g_free ( buf );
		}
		g_strfreev (sd);
		g_free (p);
	}
	gnome_config_sync ();

	/* In with the new */
	p = NULL;
	for (l = servers; l; l = l->next) {
		gchar* buf;

		sdata = (server_data *) l->data;

		buf = g_strdup_printf ( "/Pan/%s/Address", sdata->name );
		gnome_config_set_string (buf, sdata->address);
		g_free ( buf );

		buf = g_strdup_printf ( "/Pan/%s/Port", sdata->name );
		gnome_config_set_string (buf, sdata->port);
		g_free ( buf );

		buf = g_strdup_printf ( "/Pan/%s/Auth_Required", sdata->name);
		gnome_config_set_int (buf, sdata->need_auth);
		g_free ( buf );

		buf = g_strdup_printf ( "/Pan/%s/Generate_Msgid", sdata->name );
		gnome_config_set_int (buf, sdata->gen_msgid);
		g_free ( buf );

		buf = g_strdup_printf ( "/Pan/%s/Username", sdata->name );
		gnome_config_set_string (buf, sdata->username);
		g_free ( buf );

		buf = g_strdup_printf ( "/Pan/%s/Password", sdata->name );
		gnome_config_set_string (buf, sdata->password);
		g_free ( buf );

		if (p) {
			p = g_realloc (p, strlen(p) + strlen(sdata->name) + 2);
			p = strcat (p, ",");
			p = strcat (p, sdata->name);
		} else
			p = g_strdup (sdata->name);
	}

	gnome_config_set_string ("/Pan/Servers/Names", p);
	g_free (p);

	gnome_config_sync ();
}


/**
 * prefs_apply
 *
 **/
void
prefs_apply (GtkWidget *box, int page_num)
{
	gushort r, g, b, a;
	gboolean grouplist_changed = FALSE;
	gboolean articlelist_changed = FALSE;
	gboolean message_body_changed = FALSE;
	char *p;

	/* -1 means all pages, ignore anything else */
	if (page_num == -1) {
		/* Commit all of the data to the config file */

		/* No entry for these yet
		 * gnome_config_set_int("/Pan/Articles/Expiration", 30);
		 */

		gnome_config_set_string ("/Pan/User/Full_Name",
					 gtk_entry_get_text(GTK_ENTRY (prefs_window->user_fullname)));
		gnome_config_set_string ("/Pan/User/Email",
					 gtk_entry_get_text(GTK_ENTRY (prefs_window->user_email)));
		gnome_config_set_string ("/Pan/User/Organization",
					 gtk_entry_get_text(GTK_ENTRY (prefs_window->user_organization)));
		gnome_config_set_string ("/Pan/User/Signature_File",
					 gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prefs_window->user_signature_file)))));
		gnome_config_set_string ("/Pan/Paths/download_dir",
					 gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prefs_window->dir_download)))));
		gnome_config_set_string ("/Pan/Paths/temp_dir",
					 gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prefs_window->dir_temp)))));
		gnome_config_set_string ("/Pan/Paths/data_dir",
					 gtk_entry_get_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prefs_window->dir_data)))));

		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(prefs_window->headers_0)))
			gnome_config_set_int("/Pan/Message/Headers", 0);
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(prefs_window->headers_1)))
			gnome_config_set_int("/Pan/Message/Headers", 1);
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(prefs_window->headers_2)))
			gnome_config_set_int("/Pan/Message/Headers", 2);

		/* Handle color pickers */
		gnome_config_push_prefix ("/Pan/Display/");
		gnome_color_picker_get_i16(GNOME_COLOR_PICKER(prefs_window->unread_color_picker), &r, &g, &b, &a);
		if ((unread_color.red != r) || (unread_color.green != g) || (unread_color.blue != b)) {
			unread_color.red = r;
			unread_color.green = g;
			unread_color.blue = b;
			gnome_config_set_int ("unread_color_r", r);
			gnome_config_set_int ("unread_color_g", g);
			gnome_config_set_int ("unread_color_b", b);
			gnome_config_set_int ("unread_color_a", a);
			articlelist_changed = TRUE;
		}
		gnome_color_picker_get_i16(GNOME_COLOR_PICKER(prefs_window->read_color_picker), &r, &g, &b, &a);
		if ((read_color.red != r) || (read_color.green != g) || (read_color.blue != b)) {
			read_color.red = r;
			read_color.green = g;
			read_color.blue = b;
			gnome_config_set_int ("read_color_r", r);
			gnome_config_set_int ("read_color_g", g);
			gnome_config_set_int ("read_color_b", b);
			gnome_config_set_int ("read_color_a", a);
			articlelist_changed = TRUE;
		}
		gnome_color_picker_get_i16(GNOME_COLOR_PICKER(prefs_window->new_color_picker), &r, &g, &b, &a);
		if ((new_color.red != r) || (new_color.green != g) || (new_color.blue != b)) {
			new_color.red = r;
			new_color.green = g;
			new_color.blue = b;
			gnome_config_set_int ("new_color_r", r);
			gnome_config_set_int ("new_color_g", g);
			gnome_config_set_int ("new_color_b", b);
			gnome_config_set_int ("new_color_a", a);
			articlelist_changed = TRUE;
		}
		gnome_config_pop_prefix ();				 
		
		/* Handle font pickers */
		if ((p = gnome_font_picker_get_font_name (GNOME_FONT_PICKER(prefs_window->articlelist_gfp)))) {
			if (!articlelist_font || strcmp(articlelist_font, p)) {
				gnome_config_set_string ("/Pan/Display/articlelist_font", p);
				g_free (articlelist_font);
				articlelist_font = g_strdup (p);
				articlelist_changed = TRUE;	

				unread_style = NULL;
				read_style = NULL;
				new_style = NULL;
			}
		}

		if ((p = gnome_font_picker_get_font_name (GNOME_FONT_PICKER(prefs_window->grouplist_gfp)))) {
			if (!grouplist_font || strcmp(grouplist_font, p)) {
				gnome_config_set_string ("/Pan/Display/grouplist_font", p);
				g_free (grouplist_font);
				grouplist_font = g_strdup (p);
				grouplist_changed = TRUE;
			}
		}

		if ((p = gnome_font_picker_get_font_name (GNOME_FONT_PICKER(prefs_window->message_body_gfp)))) {
			if (!message_body_font || strcmp(message_body_font, p)) {
				gnome_config_set_string ("/Pan/Display/message_body_font", p);
				g_free (message_body_font);
				message_body_font = g_strdup (p);
				message_body_changed = TRUE;	
			}
		}

		if (articlelist_changed) {
			/* Make the color changes take effect */
			cmap = gdk_colormap_get_system ();
			if (!gdk_color_alloc (cmap, &unread_color))
				g_warning ("couldn't allocate color");
			if (!gdk_color_alloc (cmap, &read_color))
				g_warning ("couldn't allocate color");
			if (!gdk_color_alloc (cmap, &new_color))
				g_warning ("couldn't allocate color");
			
			articlelist_load ( Pan.active_group );
		}
		
		if (grouplist_changed)
			widget_set_font ( GTK_WIDGET(Pan.group_clist), grouplist_font );

		if (article && message_body_changed)
			widget_set_font ( GTK_WIDGET(text), message_body_font );

		save_servers ();
		gnome_config_sync ();
		server_set_active ();
		server_menu_update ();
		prefs_init ();
	}
}


void
prefs_init (void)
{
	/* Put code here that needs to be run
	 * whenever the app is started, or the
	 * properties are changed */

	/* Reset the cache of directory names */
	g_free (temp_dir);
	temp_dir = gnome_config_get_string ("/Pan/Paths/temp_dir");
	if (temp_dir == NULL)
		temp_dir = gnome_util_prepend_user_home (".pan/tmp");
	directory_check (temp_dir);

	g_free (data_dir);
	data_dir = gnome_config_get_string ("/Pan/Paths/data_dir");
	if (data_dir == NULL)
		data_dir = gnome_util_prepend_user_home (".pan/data");
	directory_check (data_dir);

	g_free (download_dir);
	download_dir = gnome_config_get_string ("/Pan/Paths/download_dir");
	if (download_dir == NULL)
		download_dir = gnome_util_prepend_user_home (".pan/download");
	directory_check (download_dir);	

/* general settings */
	online_prompt = gnome_config_get_bool ("/Pan/General/online_prompt");
	one_click_preview = gnome_config_get_bool ("/Pan/General/one_click_preview");

/* display settings */
	grouplist_font = gnome_config_get_string ("/Pan/Display/grouplist_font");
	articlelist_font = gnome_config_get_string ("/Pan/Display/articlelist_font");
	message_body_font = gnome_config_get_string ("/Pan/Display/message_body_font");

	unread_color.red = gnome_config_get_int ("/Pan/Display/unread_color_r=0");
	unread_color.green = gnome_config_get_int ("/Pan/Display/unread_color_g=0");
	unread_color.blue = gnome_config_get_int ("/Pan/Display/unread_color_b=47165");

	read_color.red = gnome_config_get_int ("/Pan/Display/read_color_r=9000");
	read_color.green = gnome_config_get_int ("/Pan/Display/read_color_g=9000");
	read_color.blue = gnome_config_get_int ("/Pan/Display/read_color_b=9000");
	
	new_color.red = gnome_config_get_int ("/Pan/Display/new_color_r=65535");
	new_color.green = gnome_config_get_int ("/Pan/Display/new_color_g=9000");
	new_color.blue = gnome_config_get_int ("/Pan/Display/new_color_b=9000");
}


/*---[ prefs_spawn ]--------------------------------------------------
 * The function that builds and draws the main prefs window.  Contains
 * a notebook with various pages for settings and such.
 *--------------------------------------------------------------------*/
void
prefs_spawn (void)
{
	/* There can be only one! (properties window) */
	if (prefs_window) {
		gdk_window_raise (GTK_WIDGET(prefs_window->box)->window);
		return;
	}
	load_servers ();

	prefs_window = (PanPrefsWindow *) g_malloc0 (sizeof(PanPrefsWindow));
	prefs_window->box = GNOME_PROPERTY_BOX (gnome_property_box_new());

	gtk_notebook_append_page (GTK_NOTEBOOK(prefs_window->box->notebook),
				  prefs_server_page(),
				  gtk_label_new (_("Server")));

	gtk_notebook_append_page (GTK_NOTEBOOK(prefs_window->box->notebook),
				  prefs_general_page(),
				  gtk_label_new (_("General")));

	gtk_notebook_append_page (GTK_NOTEBOOK(prefs_window->box->notebook),
				  prefs_userinfo_page(),
				  gtk_label_new (_("User")));

	gtk_notebook_append_page (GTK_NOTEBOOK(prefs_window->box->notebook),
				  prefs_directories_page(),
				  gtk_label_new (_("Directories")));

//	gtk_notebook_append_page (GTK_NOTEBOOK(prefs_window->box->notebook),
//				  message_page(),
//				  gtk_label_new (_("Message")));

	gtk_notebook_append_page (GTK_NOTEBOOK(prefs_window->box->notebook),
				  prefs_display_page(),
				  gtk_label_new (_("Display")));

	gtk_signal_connect (GTK_OBJECT (prefs_window->box), "apply",
			    GTK_SIGNAL_FUNC (prefs_apply), NULL);
	gtk_signal_connect (GTK_OBJECT (prefs_window->box), "destroy",
			    GTK_SIGNAL_FUNC (prefs_cancel), NULL);

	gui_popup_draw (GTK_WIDGET (prefs_window->box));
}


void
prefs_update_clist (void)
{
	char *text[2];
	GSList *l;
	server_data *sdata;
	int row;

	gtk_clist_freeze (GTK_CLIST(prefs_window->server_clist));
	gtk_clist_clear (GTK_CLIST(prefs_window->server_clist));

	for (l = servers; l; l = l->next) {
		sdata = (server_data *) l->data;
		text[0] = (sdata->name) ? sdata->name : _("Not Named");
		text[1] = (sdata->address) ? sdata->address : _("No Address");
		row = gtk_clist_prepend (GTK_CLIST(prefs_window->server_clist), text);
		gtk_clist_set_row_data (GTK_CLIST(prefs_window->server_clist), row,
					sdata);
	}

	gtk_clist_thaw (GTK_CLIST(prefs_window->server_clist));
}


void
edit_profile_dialog_clicked (GnomeDialog *dialog, int button, 
			     server_data *sdata)
{
	switch (button) {
	case 0:		/* OK button */
		if (sdata == NULL) {
			sdata = (server_data *) g_malloc0(sizeof(server_data));
			servers = g_slist_append (servers, sdata);
		}
		sdata->name = g_strdup (gtk_entry_get_text (GTK_ENTRY(prefs_window->profile_name_entry)));
		sdata->address = g_strdup (gtk_entry_get_text (GTK_ENTRY(prefs_window->server_address)));
		sdata->port = g_strdup (gtk_entry_get_text (GTK_ENTRY(prefs_window->server_port)));
//		g_print ("in edit_profile_dialog_clicked: %d\n", sdata->need_auth);
		sdata->need_auth = (!GTK_TOGGLE_BUTTON (prefs_window->auth_cbutton)->active) ? 0 : 1;
		sdata->gen_msgid = (!GTK_TOGGLE_BUTTON (prefs_window->msgid_cbutton)->active) ? 0 : 1;
		sdata->username = g_strdup (gtk_entry_get_text (GTK_ENTRY(prefs_window->server_username)));
		sdata->password = g_strdup (gtk_entry_get_text (GTK_ENTRY(prefs_window->server_password)));

//		if ((strcmp(Pan.active_server->name, sdata->name) == 0) && 
//		    (strcmp(Pan.active_server->address, sdata->address) == 0))
//		    Pan.active_server = sdata;

		prefs_update_clist ();
		gnome_property_box_changed (prefs_window->box);
//		server_set_active ();
		gtk_widget_destroy (GTK_WIDGET(dialog));
		break;
	default:	/* Cancel Button */
		gtk_widget_destroy (GTK_WIDGET(dialog));
		break;
	}
}


void
edit_profile_changed (GnomeDialog *d)
{
	gnome_dialog_set_sensitive (d, 0, TRUE);
}


static
void
auth_cbutton_set (
	GtkWidget* widget,
	server_data* sdata )
{
        if (!GTK_TOGGLE_BUTTON (widget)->active) {
                gtk_widget_set_sensitive (prefs_window->server_username_label,0);
                gtk_widget_set_sensitive (prefs_window->server_username,0);
                gtk_widget_set_sensitive (prefs_window->server_password_label,0);
                gtk_widget_set_sensitive (prefs_window->server_password,0);
		if (sdata) sdata->need_auth = 0;
        }
        else
        {
                /* disable everything */
                gtk_widget_set_sensitive (prefs_window->server_username_label,1);
                gtk_widget_set_sensitive (prefs_window->server_username,1);
                gtk_widget_set_sensitive (prefs_window->server_password_label,1);
			gtk_widget_set_sensitive (prefs_window->server_password,1);
			if (sdata) sdata->need_auth = 1;
        }
}


/*---[ edit_profile_dialog ]------------------------------------------
 * The dialog for editing a server profile.
 *
 * FIXME: changing the 'profile' entry for any server segfaults Pan
 *--------------------------------------------------------------------*/
void
edit_profile_dialog (server_data *sdata)
{
	GtkWidget *dialog;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *hbox;

	dialog = gnome_dialog_new (_("New/Edit Server"),
				   GNOME_STOCK_BUTTON_OK,
				   GNOME_STOCK_BUTTON_CANCEL, NULL);

	gnome_dialog_set_sensitive (GNOME_DIALOG(dialog), 0, FALSE);

	gtk_signal_connect (GTK_OBJECT(dialog), "clicked",
			    GTK_SIGNAL_FUNC (edit_profile_dialog_clicked),
			    sdata);

	table = gtk_table_new (4, 3, FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);
        gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
        gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	gtk_container_add (GTK_CONTAINER(GNOME_DIALOG(dialog)->vbox), table);


	frame = gtk_frame_new (_("Server Information"));
	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
	gtk_container_add (GTK_CONTAINER (frame), vbox);

	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  /* Profile Name */
	label = gtk_label_new (_("Profile Name"));
	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

	prefs_window->profile_name_entry = gtk_entry_new_with_max_length(32);
	if (sdata && sdata->name)
		gtk_entry_set_text (GTK_ENTRY(prefs_window->profile_name_entry),
				    sdata->name);
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->profile_name_entry),
				   "changed",
				   GTK_SIGNAL_FUNC(edit_profile_changed),
				   GTK_OBJECT(dialog));
	gtk_box_pack_start (GTK_BOX (hbox), prefs_window->profile_name_entry, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  /* Server Address */
	label = gtk_label_new (_("Server Address"));
	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

	prefs_window->server_address = gtk_entry_new_with_max_length (32);
	if (sdata && sdata->address)
		gtk_entry_set_text (GTK_ENTRY(prefs_window->server_address),
				    sdata->address);
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->server_address),
				   "changed",
				   GTK_SIGNAL_FUNC(edit_profile_changed),
				   GTK_OBJECT(dialog));
	gtk_box_pack_start (GTK_BOX (hbox), prefs_window->server_address, FALSE, FALSE, 0);

	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  /* Server Port */
	label = gtk_label_new (_("Server Port"));
	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

	prefs_window->server_port = gtk_entry_new_with_max_length (32);
	if (sdata && sdata->port)
		gtk_entry_set_text (GTK_ENTRY(prefs_window->server_port), sdata->port);
	else
		gtk_entry_set_text (GTK_ENTRY(prefs_window->server_port), "119");

	gtk_signal_connect_object (GTK_OBJECT(prefs_window->server_port),
				   "changed",
				   GTK_SIGNAL_FUNC(edit_profile_changed),
				   GTK_OBJECT(dialog));
	gtk_box_pack_start (GTK_BOX (hbox), prefs_window->server_port, FALSE, FALSE, 0);

	gtk_table_attach (GTK_TABLE(table), frame, 
			  0, 2, 0, 3,
			  GTK_EXPAND | GTK_FILL,
			  GTK_EXPAND | GTK_FILL,
			  0, 0);

/* Authorization frame */
	frame = gtk_frame_new (_("Authorization"));
	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
	gtk_container_add (GTK_CONTAINER (frame), vbox);

/* needs autorization information checkbox */

	prefs_window->auth_cbutton = gtk_check_button_new_with_label (_("My server requires my username and password"));
	if (sdata && sdata->need_auth)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_window->auth_cbutton), TRUE);
	else {
	}

	gtk_signal_connect (GTK_OBJECT (prefs_window->auth_cbutton), "toggled",
			    GTK_SIGNAL_FUNC (auth_cbutton_set), sdata);
	gtk_signal_connect_object (GTK_OBJECT (prefs_window->auth_cbutton), "toggled",
				   GTK_SIGNAL_FUNC (edit_profile_changed),
				   GTK_OBJECT (dialog));
	gtk_box_pack_start (GTK_BOX (vbox), prefs_window->auth_cbutton, FALSE, FALSE, 0);


	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  /* Server Username */
	prefs_window->server_username_label = gtk_label_new (_("Username"));
	gtk_box_pack_start (GTK_BOX (hbox), prefs_window->server_username_label, TRUE, TRUE, 0);
	gtk_misc_set_alignment (GTK_MISC (prefs_window->server_username_label), 1.0, 0.5);

	prefs_window->server_username = gtk_entry_new_with_max_length (32);
	if (sdata && sdata->username)
		gtk_entry_set_text (GTK_ENTRY(prefs_window->server_username),
				    sdata->username);
	if (sdata && sdata->need_auth)
		gtk_widget_set_sensitive (prefs_window->server_username, 1);
	else {
		gtk_widget_set_sensitive (GTK_WIDGET (prefs_window->server_username_label), 0);
		gtk_widget_set_sensitive (prefs_window->server_username, 0);
	}

	gtk_signal_connect_object (GTK_OBJECT(prefs_window->server_username),
				   "changed",
				   GTK_SIGNAL_FUNC(edit_profile_changed),
				   GTK_OBJECT(dialog));
	gtk_box_pack_start (GTK_BOX (hbox), prefs_window->server_username, FALSE, FALSE, 0);


  /* Server Password */
	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

	prefs_window->server_password_label = gtk_label_new (_("Password"));
	gtk_box_pack_start (GTK_BOX (hbox), prefs_window->server_password_label, TRUE, TRUE, 0);
//	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (prefs_window->server_password_label), 1.0, 0.5);

	prefs_window->server_password = gtk_entry_new_with_max_length (32);
	gtk_entry_set_visibility (GTK_ENTRY(prefs_window->server_password),
				  FALSE);
	if (sdata && sdata->password)
		gtk_entry_set_text (GTK_ENTRY(prefs_window->server_password),
				    sdata->password);
	if (sdata && sdata->need_auth)
		gtk_widget_set_sensitive (prefs_window->server_password, 1);
	else {
		gtk_widget_set_sensitive (GTK_WIDGET (prefs_window->server_password_label), 0);
		gtk_widget_set_sensitive (prefs_window->server_password, 0);
	}

	gtk_signal_connect_object (GTK_OBJECT(prefs_window->server_password),
				   "changed",
				   GTK_SIGNAL_FUNC(edit_profile_changed),
				   GTK_OBJECT(dialog));

	gtk_box_pack_start (GTK_BOX (hbox), prefs_window->server_password, FALSE, FALSE, 0);

	gtk_table_attach (GTK_TABLE(table), frame, 
			  0, 2, 4, 6,
			  GTK_EXPAND | GTK_FILL,
			  GTK_EXPAND | GTK_FILL,
			  0, 0);
	
	/* misc frame*/
        frame = gtk_frame_new (_("Misc"));
        vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
        hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
        gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
        gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
        gtk_container_add (GTK_CONTAINER (frame), vbox);

        /*Generate Message-ID*/
        prefs_window->msgid_cbutton = gtk_check_button_new_with_label (_("Generate Message-ID"));
        if (sdata && sdata->gen_msgid) {
                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_window->msgid_cbutton), TRUE);
	}
        else {
        }
        
        gtk_signal_connect_object (GTK_OBJECT (prefs_window->msgid_cbutton), "toggled",
                                   GTK_SIGNAL_FUNC (edit_profile_changed),
                                   GTK_OBJECT (dialog));
        
        gtk_box_pack_start (GTK_BOX (vbox), prefs_window->msgid_cbutton, FALSE, FALSE, 0);
        gtk_table_attach (GTK_TABLE(table), frame,
                          0, 2, 7, 8,
                          GTK_EXPAND | GTK_FILL,
                          GTK_EXPAND | GTK_FILL,
                          0, 0);

	gtk_widget_show_all (dialog);
}


void
server_clist_button_press (GtkWidget *widget, GdkEventButton *bevent)
{
	int row;
	int column;
	server_data *sdata = NULL;

	if (bevent->button != 1)
		gtk_signal_emit_stop_by_name (GTK_OBJECT(widget),
					      "button_press_event");

	if ((bevent->button) == 1 && (bevent->type == GDK_2BUTTON_PRESS)) {
		gtk_signal_emit_stop_by_name (GTK_OBJECT(widget),
					      "button_press_event");
		gtk_clist_get_selection_info (GTK_CLIST(widget), bevent->x,
					      bevent->y, &row, &column);
		sdata = (server_data *) gtk_clist_get_row_data (GTK_CLIST(widget), row);
		edit_profile_dialog (sdata);
	}
}


void
new_server_cb (void)
{
	edit_profile_dialog (NULL);
}


void
edit_server_cb (void)
{
	GList *list;
	server_data *data;

	if ((list = GTK_CLIST(prefs_window->server_clist)->selection) == NULL)
		return;
	data = (server_data *) gtk_clist_get_row_data(GTK_CLIST(prefs_window->server_clist),
						      GPOINTER_TO_INT(list->data));
	edit_profile_dialog (data);
}


void
delete_server_cb (void)
{
	GList *l;
	server_data *data;

	if ((l = GTK_CLIST(prefs_window->server_clist)->selection) == NULL)
		return;
	data = (server_data *)
		gtk_clist_get_row_data (GTK_CLIST(prefs_window->server_clist),
					GPOINTER_TO_INT(l->data));
	servers = g_slist_remove (servers, data);
	g_free (data);

	prefs_update_clist ();
	gnome_property_box_changed (prefs_window->box);
}


//void
//default_server_cb (void)
//{
//	GList *l;
//	server_data *data;
//
//	l = GTK_CLIST(prefs_window->server_clist)->selection;
//
//	data = (server_data *) gtk_clist_get_row_data (GTK_CLIST (prefs_window->server_clist), GPOINTER_TO_INT (l->data));
//
//FIXME if this code is ever made live again, remember to update the active server db here.
//better yet, just call set_active_server() from here 
//	Pan.active_server = data;
//	prefs_update_clist ();
//	gnome_property_box_changed (prefs_window->box);
//}


void
prefs_cancel (void)
{
	gtk_widget_destroy (GTK_WIDGET(prefs_window->box));
	g_free (prefs_window);
	prefs_window = NULL;
	g_slist_foreach (servers, (GFunc) server_free, NULL);
	g_slist_free (servers);
	servers = NULL;
}


void
online_cbutton_toggled (GtkWidget *widget, gpointer data)
{
	if (GTK_TOGGLE_BUTTON (widget)->active) {
		online_prompt = 1;
		gnome_config_set_bool ("/Pan/General/online_prompt", TRUE);
	}
	else {
		online_prompt = 0;
		gnome_config_set_bool ("/Pan/General/online_prompt", FALSE);
	}
}


void
one_click_preview_cbutton_toggled (GtkWidget *widget, gpointer data)
{
	if (GTK_TOGGLE_BUTTON (widget)->active) {
		one_click_preview = 1;
		gnome_config_set_bool ("/Pan/General/one_click_preview", TRUE);
	}
	else {
		one_click_preview = 0;
		gnome_config_set_bool ("/Pan/General/one_click_preview", FALSE);
	}
}


/*--------------------------------------------------------------------
 * 'general' properties: toggles and such that relate to general
 * functionality and operation of Pan
 *--------------------------------------------------------------------*/
static GtkWidget*
prefs_general_page (void)
{
	GtkWidget *table;
	table = gtk_table_new (1, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);

	/* go online dialog toggle */
	prefs_window->online_cbutton = gtk_check_button_new_with_label (_("Ask to go online if a connection is needed"));
	if (online_prompt)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_window->online_cbutton), TRUE);
		
	gtk_signal_connect (GTK_OBJECT (prefs_window->online_cbutton), "toggled",
			    GTK_SIGNAL_FUNC (online_cbutton_toggled), NULL);

	gtk_table_attach (GTK_TABLE (table), prefs_window->online_cbutton, 0, 1, 0,
			  1, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	/* single click previewing toggle */
	prefs_window->one_click_preview_cbutton = gtk_check_button_new_with_label (_("Use single click message previewing (in paned layout)"));
	if (one_click_preview)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_window->one_click_preview_cbutton), TRUE);

	gtk_signal_connect (GTK_OBJECT (prefs_window->one_click_preview_cbutton), "toggled",
			    GTK_SIGNAL_FUNC (one_click_preview_cbutton_toggled), NULL);

	gtk_table_attach (GTK_TABLE (table), prefs_window->one_click_preview_cbutton, 0, 1, 1,
			  2, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	return table;
}

/*---[ prefs_display_page ]-------------------------------------------
 * Display preferences to change things like fonts and colors.
 *--------------------------------------------------------------------*/
GtkWidget *
prefs_display_page (void)
{
	GtkWidget *table;
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *w;
	gint header;
	gchar *p;

	/* initialize the variables */
	prefs_window->grouplist_gfp = gnome_font_picker_new ();
	prefs_window->articlelist_gfp = gnome_font_picker_new ();
	prefs_window->message_body_gfp = gnome_font_picker_new ();

	prefs_window->unread_color_picker = gnome_color_picker_new ();
	prefs_window->read_color_picker = gnome_color_picker_new ();
	prefs_window->new_color_picker = gnome_color_picker_new ();

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
	hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);


/**
 * fonts frame
 **/
	table = gtk_table_new (4, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	frame = gtk_frame_new (_("Fonts"));

   /* grouplist font */
	if ((p = gnome_config_get_string ("/Pan/Display/grouplist_font")) != NULL) {
		gnome_font_picker_set_font_name (GNOME_FONT_PICKER (prefs_window->grouplist_gfp), p);
		g_free (p);
	}

	gnome_font_picker_set_mode (GNOME_FONT_PICKER (prefs_window->grouplist_gfp),
				    GNOME_FONT_PICKER_MODE_FONT_INFO);
	gnome_font_picker_fi_set_use_font_in_label (GNOME_FONT_PICKER (prefs_window->grouplist_gfp),
						    TRUE,
						    DISPLAY_PREFS_PREVIEW_FONT_SIZE);
	gtk_signal_connect_object (GTK_OBJECT (prefs_window->grouplist_gfp), "font_set",
				   GTK_SIGNAL_FUNC (gnome_property_box_changed),
				   GTK_OBJECT (prefs_window->box));

	w = gtk_label_new (_("Group List"));
	gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, 0);
	gtk_table_attach (GTK_TABLE (table), prefs_window->grouplist_gfp, 1, 2, 0, 1,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, GNOME_PAD_SMALL, 0);

	/* articlelist font */
	if ((p = gnome_config_get_string ("/Pan/Display/articlelist_font")) != NULL) {
		gnome_font_picker_set_font_name (GNOME_FONT_PICKER (prefs_window->articlelist_gfp), p);
		g_free (p);
	}
//	else
//		gnome_font_picker_set_font_name (GNOME_FONT_PICKER (prefs_window->articlelist_gfp), "-adobe-helvetica-medium-r-normal-*-*-100-*-*-p-*-iso8859-1");

	gnome_font_picker_set_mode (GNOME_FONT_PICKER (prefs_window->articlelist_gfp),
				    GNOME_FONT_PICKER_MODE_FONT_INFO);
	gnome_font_picker_fi_set_use_font_in_label (GNOME_FONT_PICKER (prefs_window->articlelist_gfp),
						    TRUE,
						    DISPLAY_PREFS_PREVIEW_FONT_SIZE);
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->articlelist_gfp), "font_set",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));

	w = gtk_label_new (_("Article List"));
	gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, 0);
	gtk_table_attach (GTK_TABLE (table), prefs_window->articlelist_gfp, 1, 2, 1, 2,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, GNOME_PAD_SMALL, 0);

	/* message body font */
	if ((p = gnome_config_get_string ("/Pan/Display/message_body_font")) != NULL) {
		gnome_font_picker_set_font_name (GNOME_FONT_PICKER (prefs_window->message_body_gfp), p);
		g_free (p);
	}

	gnome_font_picker_set_mode (GNOME_FONT_PICKER (prefs_window->message_body_gfp),
				    GNOME_FONT_PICKER_MODE_FONT_INFO);
	gnome_font_picker_fi_set_use_font_in_label (GNOME_FONT_PICKER(prefs_window->message_body_gfp),
						    TRUE,
						    DISPLAY_PREFS_PREVIEW_FONT_SIZE);

	gtk_signal_connect_object (GTK_OBJECT(prefs_window->message_body_gfp), "font_set",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));

	w = gtk_label_new (_("Message Body"));
	gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 2, 3,
			  GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, 0);
	gtk_table_attach (GTK_TABLE (table), prefs_window->message_body_gfp, 1, 2, 2, 3,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, GNOME_PAD_SMALL, 0);

	gtk_container_add (GTK_CONTAINER (frame), table);
	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, GNOME_PAD_SMALL);

/**
 * colors frame
 **/
	frame = gtk_frame_new (_("Colors"));
	table = gtk_table_new (2, 4, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	/* unread article */
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (prefs_window->unread_color_picker),
				    unread_color.red,
				    unread_color.green,
				    unread_color.blue,
				    gnome_config_get_int ("/Pan/Display/unread_color_a=65534"));
	w = gtk_label_new (_("Unread Message"));
	gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, 0);
	gtk_table_attach (GTK_TABLE (table), prefs_window->unread_color_picker, 1, 2, 0, 1,
			  GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, 0);
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->unread_color_picker), "color_set",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));

	/* read article */
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (prefs_window->read_color_picker),
				    read_color.red,
				    read_color.green,
				    read_color.blue,
				    gnome_config_get_int ("/Pan/Display/read_color_a=65534"));
	w = gtk_label_new (_("Read Message"));
	gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	gtk_table_attach (GTK_TABLE (table), prefs_window->read_color_picker, 1, 2, 1, 2,
			  GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->read_color_picker), "color_set",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));

	/* new article */
	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (prefs_window->new_color_picker),
				    new_color.red,
				    new_color.green,
				    new_color.blue,
				    gnome_config_get_int ("/Pan/Display/new_color_a=65534"));
	w = gtk_label_new (_("New Message"));
	gtk_misc_set_alignment (GTK_MISC (w), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE (table), w, 0, 1, 2, 3,
			  GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	gtk_table_attach (GTK_TABLE (table), prefs_window->new_color_picker, 1, 2, 2, 3,
			  GTK_FILL, GTK_FILL, GNOME_PAD_SMALL, GNOME_PAD_SMALL);
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->new_color_picker), "color_set",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));

	gtk_container_add (GTK_CONTAINER (frame), table);
	gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 0);

/**
 * headers frame?
 **/
	table = gtk_table_new (3, 1, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);

	frame = gtk_frame_new (_("Headers"));

	header = gnome_config_get_int ("/Pan/Message/Headers=1");

	prefs_window->headers_0 = gtk_radio_button_new_with_label (NULL, _("Brief"));
	gtk_table_attach (GTK_TABLE(table), prefs_window->headers_0, 0, 1,
			  0, 1, GTK_FILL, GTK_FILL, 4, 0);

	prefs_window->headers_1 = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(prefs_window->headers_0)), 
								   _("Partial"));
	gtk_table_attach (GTK_TABLE(table), prefs_window->headers_1, 0, 1,
			  1, 2, GTK_FILL, GTK_FILL, 4, 0);

	prefs_window->headers_2 = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(prefs_window->headers_1)),
								  _("Full"));
	gtk_table_attach (GTK_TABLE(table), prefs_window->headers_2, 0, 1,
			  2, 3, GTK_FILL, GTK_FILL, 4, 0);

	switch (header) {
	case 0:	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(prefs_window->headers_0), TRUE);
		break;
	case 1: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(prefs_window->headers_1), TRUE);
		break;
	case 2: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(prefs_window->headers_2), TRUE);
		break;
	default:
		break;
	}

	gtk_signal_connect_object (GTK_OBJECT(prefs_window->headers_0), "clicked",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->headers_1), "clicked",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->headers_2), "clicked",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));

	gtk_container_add (GTK_CONTAINER (frame), table);
	gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 0);


	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
	return vbox;
}


/*---[ prefs_directories_page ]---------------------------------------
 * Directory preferences page for setting the data, temp, and download
 * dirs in Pan.
 *--------------------------------------------------------------------*/
GtkWidget *
prefs_directories_page (void)
{
	GtkWidget *table;
	GtkWidget *label;
	gchar *p;

	table = gtk_table_new (3, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);

	label = gtk_label_new (_("Download Directory:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, 0, 0);

	prefs_window->dir_download = gnome_file_entry_new ("Prefs", _("Download Directory"));
	if ((p = gnome_config_get_string ("/Pan/Paths/download_dir")) == NULL)
		p = gnome_util_prepend_user_home (".pan/download");
	gtk_entry_set_text (GTK_ENTRY(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (prefs_window->dir_download))), p);
	gnome_file_entry_set_directory (GNOME_FILE_ENTRY (prefs_window->dir_download), TRUE);

	g_free (p);
	gtk_signal_connect_object (GTK_OBJECT(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (prefs_window->dir_download))),
				   "changed",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT (prefs_window->box));
	gtk_table_attach (GTK_TABLE(table), prefs_window->dir_download, 1, 2, 0, 1,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new (_("Temporary Directory:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, 0, 0);

	prefs_window->dir_temp = gnome_file_entry_new ("Prefs", _("Temporary Directory"));
	if ((p = gnome_config_get_string("/Pan/Paths/temp_dir")) == NULL) {
		p = gnome_util_prepend_user_home(".pan/tmp");
	}
	gtk_entry_set_text (GTK_ENTRY(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (prefs_window->dir_temp))), p);
	gnome_file_entry_set_directory (GNOME_FILE_ENTRY (prefs_window->dir_temp), TRUE);
	g_free (p);
	gtk_signal_connect_object (GTK_OBJECT(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (prefs_window->dir_temp))),
				   "changed",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT (prefs_window->box));
	gtk_table_attach (GTK_TABLE(table), prefs_window->dir_temp, 1, 2, 1,
			  2, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

/* Data Directory: */
	label = gtk_label_new (_("Data Directory:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 2, 3,
			  GTK_FILL, GTK_FILL, 0, 0);

	prefs_window->dir_data = gnome_file_entry_new ("Prefs", _("Data Directory"));
	if ((p = gnome_config_get_string("/Pan/Paths/data_dir")) == NULL) {
		p = gnome_util_prepend_user_home(".pan/data");
	}
	gtk_entry_set_text (GTK_ENTRY(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (prefs_window->dir_data))), p);
	gnome_file_entry_set_directory (GNOME_FILE_ENTRY (prefs_window->dir_data), TRUE);
	g_free (p);
	gtk_signal_connect_object (GTK_OBJECT(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (prefs_window->dir_data))),
				   "changed",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT (prefs_window->box));
	gtk_table_attach (GTK_TABLE(table), prefs_window->dir_data, 1, 2, 2,
			  3, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	return table;
}


/*---[ prefs_userinfo_page ]------------------------------------------
 * page that holds the user information such as full name,
 * e-mail, organization, and signature file (this page is optional).
 *--------------------------------------------------------------------*/
GtkWidget *
prefs_userinfo_page (void)
{
	GtkWidget *table;
	GtkWidget *label;
	//GtkWidget *frame;
	char *p;

	table = gtk_table_new (4, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
	gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD_SMALL);
	gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);

	//frame = gtk_frame_new (_("Personal Information"));

	label = gtk_label_new (_("Full Name:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 0, 1,
			  GTK_FILL, GTK_FILL, 0, 0);

	prefs_window->user_fullname = gtk_entry_new ();
	if ((p = gnome_config_get_string("/Pan/User/Full_Name"))) {
		gtk_entry_set_text (GTK_ENTRY(prefs_window->user_fullname), p);
		g_free(p);
	}
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->user_fullname),
				   "changed",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));
	gtk_widget_show (prefs_window->user_fullname);
	gtk_table_attach (GTK_TABLE(table), prefs_window->user_fullname, 1, 2, 0, 1,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new (_("Email Address:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 1, 2,
			  GTK_FILL, GTK_FILL, 0, 0);

	prefs_window->user_email = gtk_entry_new ();
	if ((p = gnome_config_get_string("/Pan/User/Email"))) {
		gtk_entry_set_text (GTK_ENTRY(prefs_window->user_email), p);
		g_free(p);
	}
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->user_email),
				   "changed",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));
	gtk_widget_show (prefs_window->user_email);
	gtk_table_attach (GTK_TABLE(table), prefs_window->user_email, 1, 2, 1, 2,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new (_("Organization:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 2, 3,
			  GTK_FILL, GTK_FILL, 0, 0);

	prefs_window->user_organization = gtk_entry_new ();
	if ((p = gnome_config_get_string("/Pan/User/Organization"))) {
		gtk_entry_set_text (GTK_ENTRY(prefs_window->user_organization), p);
		g_free(p);
	}
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->user_organization),
				   "changed",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));
	gtk_widget_show (prefs_window->user_organization);
	gtk_table_attach (GTK_TABLE(table), prefs_window->user_organization, 1,
			  2, 2, 3, GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

	label = gtk_label_new (_("Signature File:"));
	gtk_misc_set_alignment (GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach (GTK_TABLE(table), label, 0, 1, 3, 4,
			  GTK_FILL, GTK_FILL, 0, 0);

	prefs_window->user_signature_file = gnome_file_entry_new ("Prefs", _("Signature File:"));
	if ((p = gnome_config_get_string("/Pan/User/Signature_File"))) {
		gtk_entry_set_text (GTK_ENTRY(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (prefs_window->user_signature_file))), p);
		g_free(p);
	}
	gtk_signal_connect_object (GTK_OBJECT(gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (prefs_window->user_signature_file))),
				   "changed",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));
	gtk_widget_show (GTK_WIDGET (prefs_window->user_signature_file));
	gtk_table_attach (GTK_TABLE(table), GTK_WIDGET (prefs_window->user_signature_file), 1, 2, 3, 4,
			  GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);
	return table;
}


GtkWidget *
message_page (void)
{
	GtkWidget *table;
	GtkWidget *w;
	gint header;
	
	table = gtk_table_new (3, 2, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE(table), 2);
	gtk_table_set_col_spacings (GTK_TABLE(table), 2);

	header = gnome_config_get_int ("/Pan/Message/Headers=1"); 
	
	w = gtk_label_new (_("Header style: "));

	gtk_table_attach (GTK_TABLE(table), w, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 4, 0);
    
	prefs_window->headers_0 = gtk_radio_button_new_with_label (NULL, _("Brief"));
	gtk_table_attach (GTK_TABLE(table), prefs_window->headers_0, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 4, 0);

	gtk_signal_connect_object (GTK_OBJECT(prefs_window->headers_0), "clicked",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));

	prefs_window->headers_1 = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(prefs_window->headers_0)), 
								   _("Partial"));
	gtk_table_attach (GTK_TABLE(table), prefs_window->headers_1, 1,
			  2, 1, 2, GTK_FILL, GTK_FILL, 4, 0);

	gtk_signal_connect_object (GTK_OBJECT(prefs_window->headers_1), "clicked",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));
	
	prefs_window->headers_2 = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(prefs_window->headers_1)),
								  _("Full"));
	gtk_table_attach (GTK_TABLE(table), prefs_window->headers_2, 1,
			  2, 2, 3, GTK_FILL, GTK_FILL, 4, 0);
	gtk_signal_connect_object (GTK_OBJECT(prefs_window->headers_2), "clicked",
				   GTK_SIGNAL_FUNC(gnome_property_box_changed),
				   GTK_OBJECT(prefs_window->box));

	switch (header) {
	case 0:	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(prefs_window->headers_0), TRUE);
		break;
	case 1: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(prefs_window->headers_1), TRUE);
		break;
	case 2: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(prefs_window->headers_2), TRUE);
		break;
	default:
		break;
	}

	return table;
}


/*---[ prefs_server_page ]--------------------------------------------
 * This is the Server tab of the preferences.  It has a list of the
 * server's and buttons that allow users to make a new server entry,
 * edit a previous one, or delete one.  This doesn't actually let
 * them switch servers, that's done from the main menu of Pan.
 *--------------------------------------------------------------------*/
GtkWidget *
prefs_server_page (void)
{
	GtkWidget *vbox, *hbox, *button;

	char *titles[] = {
		_("Name"),
		_("Address"),
	};

	hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox);

	prefs_window->server_clist = gtk_clist_new_with_titles (2, titles);
	gtk_clist_column_titles_passive (GTK_CLIST(prefs_window->server_clist));
	gtk_clist_set_selection_mode (GTK_CLIST(prefs_window->server_clist),
				      GTK_SELECTION_SINGLE);

//	gtk_clist_set_column_width (GTK_CLIST(prefs_window->server_clist), 0, 200);
//	gtk_clist_set_column_width (GTK_CLIST(prefs_window->server_clist), 1, 40);
	gtk_clist_set_column_width (GTK_CLIST(prefs_window->server_clist), 0, 100);
	gtk_clist_set_column_auto_resize (GTK_CLIST(prefs_window->server_clist), 1, TRUE);

	gtk_widget_set_usize (prefs_window->server_clist, 280, 200);
	gtk_box_pack_start (GTK_BOX(hbox), prefs_window->server_clist, TRUE,
			    TRUE, 5);
	gtk_signal_connect (GTK_OBJECT(prefs_window->server_clist),
			    "button_press_event",
			    GTK_SIGNAL_FUNC(server_clist_button_press), NULL);

	prefs_update_clist ();

	/* one button vbox */
	vbox = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), vbox, FALSE, FALSE, 5);

	/* new server button */
	button = gtk_button_new_with_label (_("New..."));
	gtk_box_pack_start (GTK_BOX(vbox), button, FALSE, FALSE, GNOME_PAD);
	gtk_signal_connect (GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC (new_server_cb), NULL);

	/* edit server button */
	button = gtk_button_new_with_label (_("Edit..."));
	gtk_box_pack_start (GTK_BOX(vbox), button, FALSE, FALSE, GNOME_PAD);
	gtk_signal_connect (GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC (edit_server_cb), NULL);

	/* delete server button */
	button = gtk_button_new_with_label(_("Delete"));
	gtk_box_pack_start (GTK_BOX(vbox), button, FALSE, FALSE, GNOME_PAD);
	gtk_signal_connect (GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC (delete_server_cb), NULL);

/* make default button */
//	button = gtk_button_new_with_label ("Make Default");
//	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, GNOME_PAD);
//	gtk_signal_connect (GTK_OBJECT (button), "clicked",
//			    GTK_SIGNAL_FUNC (default_server_cb), NULL);

	return hbox;
}
