/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for mo0re details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include "pan.h"

static GSList *queue = NULL;

static pthread_mutex_t qlock = PTHREAD_MUTEX_INITIALIZER;
static pthread_mutex_t qcond_m = PTHREAD_MUTEX_INITIALIZER;
static pthread_cond_t qcond = PTHREAD_COND_INITIALIZER;

static gint qtotal = 1;

static void queue_run_decode ( queue_data* );
static void queue_run_article ( queue_data* );
static void queue_prepend ( queue_data* );
static void queue_processing ( void );


int
queue_active (int set)
{
	static int qactive;

	qactive = set ? 1 : 0;

	return qactive;
}


void
queue_add_article_download (server_data* sdata,
			    group_data* gdata,
			    const char* message_id,
			    guint flags)
{
	/* create the request */
	queue_data* qdata = g_new0 ( queue_data, 1 );
	qdata->sdata = sdata;
	qdata->gdata = gdata;
	qdata->ptr = g_strdup ( message_id );
	qdata->flags = QUEUE_ARTICLE_DOWNLOAD | flags;

	/* queue the request */
	pthread_mutex_lock (&qlock);
	if (flags & QUEUE_FOREGROUND)
		queue_prepend (qdata);
	else if (flags & QUEUE_BACKGROUND)
		queue_append (qdata);
	else
		g_error ("Unreachable condition reached\n");
	pthread_mutex_unlock (&qlock);

	/* Signal the addition */
	pthread_mutex_lock (&qcond_m);
	pthread_cond_broadcast (&qcond);
	pthread_mutex_unlock (&qcond_m);
}


void
queue_add_articlelist_download (group_data *gdata, guint flags)
{
	queue_data *qdata;

	qdata = g_new0 ( queue_data, 1 );
	qdata->gdata = gdata;
	qdata->flags = flags;

	/* queue the request */
	pthread_mutex_lock (&qlock);
	if (flags & QUEUE_FOREGROUND)
		queue_prepend (qdata);
	else if (flags & QUEUE_BACKGROUND)
		queue_append (qdata);
	else
		g_error("Unreachable condition reached\n");
	pthread_mutex_unlock (&qlock);

	/* Signal the addition */
	pthread_mutex_lock(&qcond_m);
	pthread_cond_broadcast(&qcond);
	pthread_mutex_unlock(&qcond_m);
}


void
queue_add_grouplist_download (void)
{
	queue_data *qdata;

	qdata = g_new0 ( queue_data, 1 );
	qdata->flags = QUEUE_GROUPLIST_DOWNLOAD;

	/* queue the request */
	pthread_mutex_lock (&qlock);
	queue = g_slist_prepend (queue, qdata);
	pthread_mutex_unlock (&qlock);

	/* Signal the addition */
	pthread_mutex_lock (&qcond_m);
	pthread_cond_broadcast (&qcond);
	pthread_mutex_unlock (&qcond_m);
}


void
queue_add_article_decode (
	server_data* sdata,
	group_data* gdata,
	GSList* message_id_list,
	guint flags )
{
	queue_data* qdata;
	GSList* l;

	/* parameter sanity check */
	g_assert ( gdata );
	g_assert ( message_id_list!=0 );
	g_assert ( flags );

	/* create the queue entry */
	qdata = g_new0 ( queue_data, 1 );
	qdata->sdata = sdata;
	qdata->gdata = gdata;
	qdata->flags = flags;
	qdata->ptr = g_slist_copy ( message_id_list );
	for ( l=(GSList*)qdata->ptr; l!=NULL; l=l->next )
		l->data = g_strdup ( (gchar*)l->data );

	/* add this to the queue */	
	pthread_mutex_lock ( &qlock );
	queue_append ( qdata );
	pthread_mutex_unlock ( &qlock );

	/* Signal the addition */
	pthread_mutex_lock ( &qcond_m );
	pthread_cond_broadcast ( &qcond );
	pthread_mutex_unlock ( &qcond_m );
}


void
queue_add_post (
	const gchar* message )
{
	queue_data *qdata;

	qdata = (queue_data *) g_malloc0 (sizeof(queue_data));
	qdata->ptr = g_strdup ( message );
	qdata->flags = QUEUE_ARTICLE_POST;

	/* queue the request */
	pthread_mutex_lock (&qlock);
	queue_prepend (qdata);
	pthread_mutex_unlock (&qlock);

	/* Signal the addition */
	pthread_mutex_lock (&qcond_m);
	pthread_cond_broadcast (&qcond);
	pthread_mutex_unlock (&qcond_m);
}


void
queue_append (queue_data *qdata)
{
	queue = g_slist_append ( queue, qdata );
}


/*---[ queue_get_next ]-----------------------------------------------
 * return next item on queue, remove link to current one
 *--------------------------------------------------------------------*/
queue_data*
queue_get_next ( void )
{
	queue_data* retval = NULL;

	if ( queue != NULL )
	{
		pthread_mutex_lock ( &qlock );
		retval = (queue_data *) queue->data;
		queue = g_slist_remove_link ( queue, queue );
		pthread_mutex_unlock ( &qlock );
	}

	return retval;
}


/* FIXME: make me do something */
#if 0
static
queue_data*
queue_show_next ( void )
{
	return NULL;
}
#endif


static
int
queue_get_total ( void )
{
	return queue ? g_slist_length(queue) : 0;
}


void
queue_init (void)
{
	/* Fire up the queue thread */
	pthread_create (&Pan.queue_t, NULL, (void *) &queue_processing, NULL);
	pthread_detach (Pan.queue_t);
}


/*--------------------------------------------------------------------
 * put an item at the front of the queue
 *--------------------------------------------------------------------*/

static
void
queue_prepend (
	queue_data *qdata )
{
	queue = g_slist_prepend ( queue, qdata );
}


static
void
queue_processing (void)
{
	struct timeval now;
	struct timespec timeout;
	int retcode;
	queue_data *qdata;
	time_t last_run, current_time;
	
	while (1) {
		pthread_mutex_lock (&qcond_m);
		gettimeofday (&now, NULL);
		timeout.tv_sec = now.tv_sec + 20;
		timeout.tv_nsec = now.tv_usec * 1000;
		retcode = 0;
	
		/* Wait for something to hit the queue */
		retcode = pthread_cond_timedwait (&qcond, &qcond_m, &timeout);
		if ((retcode == ETIMEDOUT) && (queue == NULL)) {
			/* Nothing hit the queue before timeout */
			if (is_online()) {
				time (&current_time);
				if ((current_time - last_run) > 300)
					/* We've been idle for 300 seconds, so go offline */
					nntp_want_disconnect ();
				else
					/* Send HELP as a no-operation to keep the connection alive */
					nntp_send_noop ();
			}
		}
		else {
			/* Something hit the queue, do it */
			pthread_mutex_unlock (&qcond_m);
			queue_active (1);

			while ((qdata = queue_get_next())) {
				qtotal = queue_get_total ();
				queue_run (qdata);
			}
			/* Record the last time we ran queue */
			queue_active (0);
			pthread_mutex_lock (&qcond_m);
			time (&last_run);
		}
		pthread_mutex_unlock (&qcond_m);
	}
}


/*---[ queue_purge ]--------------------------------------------------
 * remove all items from the queue
 *--------------------------------------------------------------------*/
void
queue_purge (void)
{
	GSList *p;

	pthread_mutex_lock (&qlock);
	for (p = queue; p; p = p->next) {
		/* FIXME: Remove the "queued" icon in the article_ctree */
		g_free (p->data);
	}
	g_slist_free (queue);
	queue = NULL;

	pthread_mutex_unlock (&qlock);
}


/*
void
stop_queue_processing (void)
{
	pthread_mutex_lock(&qlock);
	g_slist_free(queue);
	queue = NULL;
	q_active = FALSE;
	pthread_mutex_unlock(&qlock);
	nntp_want_disconnect();
}
*/

static
int
queue_connect_wait ( void )
{
	while (1) {
		//pthread_mutex_lock(&online_m);
		if (online == TRUE) {
			//pthread_mutex_unlock(&online_m);
			return TRUE;
		}
		//pthread_mutex_unlock(&online_m);
		if (stop_all || must_exit) {
			return FALSE;
		}
		usleep (1000);
	}		
}


void
queue_run_grouplist (queue_data *qdata)
{
	GSList *glist = NULL;

	/* Download the new one */
	nntp_want_connect();
	if (queue_connect_wait() == FALSE)
		return;
	if ((glist = nntp_grouplist_download())) {
		check_new_group_list_for_newness (glist);
		grouplist_save (glist);
		grouplist_free (glist);
		grouplist_load ();	
		grouplist_update ();
	}
}


void
queue_run_article (queue_data *qdata)
{
	article_db db = ahdb_ref ( Pan.active_server, qdata->gdata );
	article_data *adata = ahdb_get ( db, (const char*)qdata->ptr );
	ahdb_unref ( db );

	/* make sure we've got the first header */
	if ( !adata ) {
		log_add_va (_("Couldn't find article header for message_id '%s'"), qdata->ptr);
		g_free ( qdata->ptr );
		return;
	}

	g_free ( qdata->ptr );
	article_remove_flag ( adata, STATE_DOWNLOAD_QUEUED, TRUE );

	/* Clear the one already displayed, if any */
	if (qdata->flags & QUEUE_FOREGROUND) {
		text_clear();
	}

	/* Check the cache before we download */
	if (acache_hit_header (adata->message_id) || acache_hit_body(adata->message_id)) {
		nntp_want_connect();
		if (queue_connect_wait() == FALSE)
			return;
		if (nntp_article_download (qdata->gdata, adata) != 0) {
			article_free (adata);
			return;
		}
	} 
	
	/* Display, et cetera */
	if (qdata->flags & QUEUE_FOREGROUND)
		text_update (adata->message_id);

	if (adata->state != STATE_READ) {
		group_add_attrib_i (qdata->gdata, "Read", 1);
		article_add_flag (adata, STATE_READ, TRUE);
	}

	article_free (adata);
}


static void
queue_run_decode (
	queue_data* qdata )
{
	gboolean success = FALSE;

	/* Make sure the first one is in the db */
	article_db db = ahdb_ref ( qdata->sdata, qdata->gdata );
	article_data* adata = ahdb_get ( db, ((GSList*)qdata->ptr)->data );
	ahdb_unref ( db );
	if ( !adata ) {
		gchar* pch = (gchar*) ((GSList*)qdata->ptr)->data;
		log_add_va (_("Couldn't find article header for message_id '%s'"), pch );
		return;
	}

	/* connect */
	nntp_want_connect();
	if ( queue_connect_wait() )
	{
		/* get the articles & decode them */
		if ( !nntp_article_decode ( qdata->gdata, qdata->ptr) )
		{
			/* Decode them */
			gui_set_status_va (_("Preparing article '%s' for decode..."), adata->subject);
			gui_set_progress (0);

			if ((adata->state == STATE_NONE) || (adata->state == STATE_NEW))
				group_add_attrib_i (qdata->gdata, "unread", -1);

			if (article_decode (adata, (GSList *)qdata->ptr, qdata->flags & QUEUE_OPEN ? 1:0) != -1)
				success = TRUE;
		}
	}

	/* update the article's flags */
	adata->state &= ~STATE_DECODE_QUEUED;
	article_add_flag ( adata, (success ? STATE_DECODED : STATE_DECODE_FAILED), TRUE );

	/* cleanup */
	article_free ( adata );
}


void
queue_run_articlelist (queue_data *qdata)
{
	int count = 0;
	GSList *alist = NULL;
	article_db db = NULL;
	group_data *gdata = qdata->gdata;

	/* connect... */
	nntp_want_connect();
	if ( !queue_connect_wait() )
		return;

	/* get the article headers... */
	alist = nntp_articlelist_download(gdata, qdata->flags, &count);
	if ( !alist ) {
		log_add_va (_("No articles retrieved for group %s"), gdata->name);
		return;
	}

	db = ahdb_ref ( Pan.active_server, gdata );

	log_add_va (_("Retrieved %d article headers from %s"), count, gdata->name);

	if ( qdata->flags&QUEUE_ARTICLELIST_DOWNLOAD_NEW )
	{
		/* get old headers, if necessary */
		gboolean update_ui = (qdata->flags&QUEUE_FOREGROUND) && (gdata==Pan.active_group);
		GSList* old_headers = update_ui ? ahdb_get_all(db,NULL,NULL) : 0;

		/* save the new headers */
		gui_progress_init_steps ( count );
		ahdb_save_all ( db, alist, gui_progress_next_step_gfunc, NULL );
		group_add_attrib_i (gdata, "Total", count);

		/* join new headers with any old ones.
                   note that since old ones were loaded from disk,
                   their "new" flag is turned off automatically */
		if ( update_ui )
			alist = g_slist_concat ( alist, old_headers );
		printf ( "comibined length: %d\n", g_slist_length(alist) );
	}
	else
	{
		/* Save all the fresh new headers */
		gui_progress_init_steps ( count );
		ahdb_erase_all ( db );
		ahdb_save_all ( db, alist, gui_progress_next_step_gfunc, NULL );
		group_set_attrib_i ( gdata, "Read", 0 );
		group_set_attrib_i ( gdata, "Total", count );
	}

	/* update grouplist */
	grouplist_update_row ( gdata );

	/* update articlelist */
	if ( gdata == Pan.active_group ) {
		gui_set_status (_("Threading article list"));
		alist = thread_articles ( alist );
		gui_set_status (_("Displaying article headers"));
		articlelist_update ( gdata, alist );
	}

	/* cleanup */	
	ahdb_sync ( db );
	ahdb_unref ( db );
	articlelist_free ( alist );
}


void
queue_run_post (queue_data *qdata)
{
	nntp_want_connect();
	if (queue_connect_wait() == FALSE)
		return;
	if (nntp_post((char *)qdata->ptr) == 0) {
//		gui_set_info (_("Message posted."));
		log_add_va (_("Message posted.")); /* FIXME: message '<subject here>' posted */
	}
}


void
queue_run (queue_data *qdata)
{
	g_assert (qdata);
	
	switch (qdata->flags & 0x003F) {
	case QUEUE_ARTICLE_DOWNLOAD:
		queue_run_article (qdata);
		break;
	case QUEUE_ARTICLELIST_LOAD:
		articlelist_load ( qdata->gdata );
		break;
	case QUEUE_ARTICLELIST_DOWNLOAD_ALL:
	case QUEUE_ARTICLELIST_DOWNLOAD_NEW:
	case QUEUE_ARTICLELIST_DOWNLOAD_SAMPLE:
		queue_run_articlelist (qdata);
		break;
	case QUEUE_GROUPLIST_DOWNLOAD:
		queue_run_grouplist (qdata);
		break;
	case QUEUE_ARTICLE_POST:
		queue_run_post (qdata);
		g_free ( qdata->ptr ); 
		break;
	case QUEUE_ARTICLE_DECODE:
		queue_run_decode (qdata);
		break;
	default:
                g_assert_not_reached ( );
		break;
	}

	/* Put stuff here you want to do after processing
	 * queue items */
	g_free (qdata);
	/* FIXME: queue_run_decode leaks the message_id strings in the queue ptr GSList data fields */
	gui_clean ();

#if SHOW_DEBUG_MSG
	g_print ("# of tasks in queue: %d\n", --qtotal);
	g_print ("total bytes received: %ld\n", pan_stats.bytes_received);
	g_print ("total bytes sent: %ld\n", pan_stats.bytes_sent);
	if (pan_stats.time_online > 0)
		g_print ("cumulative transfer rate: %f kbytes/sec\n", (pan_stats.bytes_received / (float) pan_stats.time_online) / 1024.0);
#endif
}
