/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <sys/stat.h>
#include <errno.h>
#include <ctype.h>
#include "pan.h"

void
pan_error_dialog (const gchar *format, ...)
{
	va_list args;
	gchar *string;

	g_return_if_fail (format != NULL);

	va_start (args, format);
	string = g_strdup_vprintf (format, args);
	va_end (args);

	pan_lock ();
	gnome_error_dialog (string);
	pan_unlock ();
	g_free (string);
}


gchar *
Pan_strtrim (gchar *string)
{
	gchar *rp, *wp;
	int ws = 1;

	for (wp = rp = string; *rp; rp++) {	/* loop over string */
		if (isspace((int)*rp)) {	/* is it a white space? */
			if (!ws) {		/* was the last character not a white space? */
				*wp++ = ' ';	/* store a blank */
				ws = 1;
			}
		} else {			/* no white space */
			*wp++ = *rp;		/* store the character */
			ws = 0;
		}
	}
	if (ws)					/* delete trailing blank */
		wp--;

	*wp = '\0';

	return (string);
}


gchar *
Pan_strbrk (gchar *str, gchar *list)
{
	gchar ch, ch1, *p;

	while ((ch = *str) != 0) {
		p = list;
		while ((ch1 = *p) != 0) {
			if (ch == ch1)
				return str;
			p++;
		}
		str++;
	}
	return NULL;
}


/*---[ not_implemented ]----------------------------------------------
 * ok dialog popup to notify user that we are so lazy we haven't
 * added their feature yet.
 *--------------------------------------------------------------------*/
void
not_implemented (void)
{
	gnome_ok_dialog (_("Not yet implemented."));
}


void
clist_unselect_all (GtkWidget *clist)
{
	GList *list;
	gint row;

	list = GTK_CLIST(clist)->selection;
	while (list) {
		row = GPOINTER_TO_INT (list->data);
		list = list->next;
		gtk_clist_unselect_row (GTK_CLIST(clist), row, 0);
	}
	g_list_free (list);
}

#if 0
gchar *
fgetline (FILE *fp)
{
	gchar *tmp = NULL;
	long pos;
	gint i;

	/* Skip Whitespace */
	for (; fgetc(fp) == ' ';);
	fseek(fp, -1, SEEK_CUR);

	/* Record String start */
	pos = ftell(fp);

	/* Measure size of string */
	for (i = 1; fgetc(fp) != '\n'; i++);

	if (i == 1)
		return NULL;

	/* Rewind to beginning of string */
	if ((fseek(fp, pos, SEEK_SET)) == -1)
		return NULL;

	/* Make room and grab that string */
	tmp = (gchar *) g_malloc (i);
	fgets (tmp, i, fp);

	/* leave the file past the newline */
	fseek (fp, 1, SEEK_CUR);

	return tmp;
}
#endif


void
pan_lock (void)
{
	pthread_t thr;

	thr = pthread_self();
	if ((thr == Pan.queue_t) || (thr == Pan.conmgr_t))
		gdk_threads_enter();
}


void
pan_unlock (void)
{
	pthread_t thr;

	thr = pthread_self();
	if ((thr == Pan.queue_t) || (thr == Pan.conmgr_t))
		gdk_threads_leave();
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
void
directory_check (gchar * pathname)
{
	struct stat st;
	gchar *dir;
	gint len, i;

	len = strlen (pathname);

	i = 1;
	while (i <= len) {
		if (pathname[i] == '/' || pathname[i] == '\0') {
			dir = g_strndup (pathname, i);
			if (stat (dir, &st) != 0) {
				int ret;
				ret = mkdir (dir, S_IRUSR | S_IWUSR | S_IXUSR);
				if (ret == -1) {
					pan_error_dialog (_("Couldn't create directory: "),
							  pathname, strerror(errno));
					return;
				}
			}
			g_free (dir);
		}
		i++;
	}
}


/**
 * http://www.strangecreations.com/library/snippets/
 * author: Fred Cole
 * ansified by Bob Stout
 * Hereby donated to public domain.
 */
char *
stristr (const char *str, const char *search )
{
      char *pptr, *sptr, *start;
      uint  slen, plen;

      for (start = (char *)str,
           pptr  = (char *)search,
           slen  = strlen(str),
           plen  = strlen(search);

           /* while string length not shorter than pattern length */

           slen >= plen;

           start++, slen--)
      {
            /* find start of pattern in string */
            while (toupper(*start) != toupper(*search))
            {
                  start++;
                  slen--;

                  /* if pattern longer than string */

                  if (slen < plen)
                        return(NULL);
            }

            sptr = start;
            pptr = (char*)search;

            while (toupper(*sptr) == toupper(*pptr))
            {
                  sptr++;
                  pptr++;

                  /* if end of pattern then pattern was found */

                  if ('\0' == *pptr)
                        return (start);
            }
      }
      return(NULL);
}

int lower_bound (
   const void* key,
   const void* base,
   size_t n,
   size_t size,
   int (*compare)(const void *, const void *),
   gboolean* exact_match )
{
   int low = 0;
   int high = n-1;

   while ( low <= high )
   {
      const int mid = (low+high)/2;
      const int comp = (*compare)(key,(const void*)(((char*)(base))+(mid*size)));
      if ( comp > 0 ) low = mid + 1;
      else if ( comp < 0 ) high = mid - 1;
      else {
         if ( exact_match!=NULL )
            *exact_match = TRUE;
         return mid;
      }
   }

   if ( exact_match!=NULL )
      *exact_match = FALSE;
   return low;
}


void
widget_set_font (GtkWidget* w,
		 const char* font_name)
{
	GtkStyle* style = NULL;
	GdkFont* font = NULL;

	g_return_if_fail ( w!=NULL );
	if ( !font_name || !*font_name )
		return;

	style = gtk_style_copy ( gtk_widget_get_style ( Pan.window ) );
	font = gdk_font_load ( font_name );
	if ( font != NULL )
		style->font = font;
	else
		g_warning ( "Couldn't load font ``%s''", font_name );
	gtk_widget_set_style ( w, style );
}


GArray*
read_file (const char* filename)
{
	GArray* array = NULL;
	unsigned long len = 0;
	FILE* fp = fopen ( filename, "r" );

	if ( !fp ) {
		pan_error_dialog ( _("Unable to open file: %s\n%s"), filename, strerror(errno) );
		return NULL;
	}

	fseek ( fp, 0, SEEK_END );
	len = ftell ( fp );
	if ( len <= 0 ) {
		pan_error_dialog ( _("There was a problem loading the file %s"), filename );
		fclose ( fp );
		return NULL;
	}

	fseek ( fp, 0, SEEK_SET );

	array = g_array_new ( TRUE, FALSE, 1 );
	g_array_set_size ( array, len );
	array->len = fread ( array->data, 1, len, fp );
	return array;
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
gchar *
commatize_ulong (gulong num)
{
	gchar *p;
	gchar *string;
	gushort len;
	gshort i = 0;
	gushort j = 0;

	string = g_strdup_printf ("%ld", num);
	len = strlen (string);

	p = g_malloc0 (len + len/3 + 1);

#if SHOW_DEBUG_MSG
	g_print ("string: %s\n", string);
	g_print ("string length: %d\n", len);
#endif

	for (i = (len-1); i >= 0; i--) {
		if (((len-1-i) > 0) && !((len-1-i) % 3)) {
			*(p+len-1-i+j) = ',';
			++j;
		}

		*(p+len-1-i+j) = (gchar) *(string + i);
	}
	g_strreverse (p);

#if SHOW_DEBUG_MSG
	g_print ("p: %s\n", p);
#endif
        g_free ( string );

	return p;
}
